/******************************************************************************/
/*                                                                            */
/*  Copyright (C)        Endress+Hauser (2002). All Rights Reserved           */
/*                                                                            */
/*  Product:             SIMATIC PDM                                          */
/*                                                                            */
/*  Device:              ECHO LEVEL DEVICE Micropilot M for HART              */
/*  Function:            Device Description Implementation for FMR2xx         */
/*  Description:         Device Description                                   */
/*  Filename:            MPM_Hart.ddl                                         */
/*  Dictionary:          EHHA__MicropilotM_00.dct                             */
/*                                                                            */
/*  Manufacturer:        Endress & Hauser GmbH & Co.                          */
/*   *                   Hauptstrasse 1                                       */
/*   *                   D-79689 Maulburg                                     */
/*   *                   Tel:07622/28-0                                       */
/*                                                                            */
/*  Author:              Stefan Mueller, Julien Messer                        */
/*   *                   Interpro GmbH                                        */
/*   *                   CH-4144 Arlesheim                                    */
/*                                                                            */
/*  Revision:            1.8                                                  */
/*  Date:                19.06.02                                             */
/*                                                                            */
/*  QVCS                 $Revision:1.8$                                       */
/*  QVCS                 $Date:19.06.02$                                      */
/*                                                                            */
/******************************************************************************/

/*----------------------DEVICE & REVISION CODES-------------------------*/

MANUFACTURER		ENDRESS_HAUSER,
DEVICE_TYPE             15,
DEVICE_REVISION		1,
DD_REVISION		3

// (JM) Note: only the following releases are allowed (see MPM_HA.inc).
#define  CHECK_SOFTWARE_VERSION    1
#define  CHECK_SOFTWARE_VERSION02  2

#define PRIMARY					1
#define SECONDARY				2
#define TERTIARY				3
#define FOURTH					4
#define PV  dynamic_variables[PRIMARY]
#define SV  dynamic_variables[SECONDARY]
#define TV  dynamic_variables[TERTIARY]
#define QV  dynamic_variables[FOURTH]
#define DEFAULT_VALUE_phys_device_id "FMR 2XX"

/************************************************************************************************************/
/*                         COMMON TABLES	 																*/
/************************************************************************************************************/

IMPORT STANDARD _TABLES, DEVICE_REVISION 10, DD_REVISION 1
{
	EVERYTHING ;
}

/************************************************************************************************************/
/*                         UNIVERSAL COMMANDS																*/
/************************************************************************************************************/

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 5, DD_REVISION 1
{
	EVERYTHING ;

    REDEFINITIONS  {
	DELETE COMMAND read_dynamic_variables_and_pv_current;
        DELETE VARIABLE tag;
        DELETE VARIABLE message;
        DELETE VARIABLE descriptor;
        DELETE VARIABLE date;
        DELETE VARIABLE manufacturer_id;
        DELETE VARIABLE universal_revision;

        DELETE VARIABLE software_revision;      // (JM): to redefine PostReadAction

        DELETE VARIABLE final_assembly_number;

        VARIABLE final_assembly_number
        {
            REDEFINE TYPE UNSIGNED_INTEGER (3)
            {
                DISPLAY_FORMAT "6u";
                EDIT_FORMAT    "6u";
            }
        }

        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED
            {
                { 15,  "FMR2xx" }
            }
        }
    }
}


COMMAND read_dynamic_variables_and_pv_current
{
	NUMBER 3;

	OPERATION READ;

	TRANSACTION
	{
		REQUEST
			{}
                REPLY
                {
                    response_code,
                    device_status,
                    PV.ANALOG_VALUE,
                    PV.DIGITAL_UNITS,
                    PV.DIGITAL_VALUE
                }
		RESPONSE_CODES
		{
				0, SUCCESS,          [no_command_specific_errors] ;
				2, DATA_ENTRY_ERROR, [invalid_selection] ;
				3, DATA_ENTRY_ERROR, [passed_parameter_too_large] ;
				4, DATA_ENTRY_ERROR, [passed_parameter_too_small] ;
				5, MISC_ERROR,       [too_few_data_bytes_recieved] ;
				7, MODE_ERROR,       [in_write_protect_mode] ;
				8, MISC_WARNING,     "Set to nearest possible value" ;
				9, DATA_ENTRY_ERROR, "Invalid value selection" ;
				16, MODE_ERROR,       [access_restricted] ;
		}
        }
}


/************************************************************************************************************/
/*                         COMMON PRACTICE COMMANDS															*/
/************************************************************************************************************/

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 7, DD_REVISION 2
{


	VARIABLE	xmtr_specific_status_0 ;
	VARIABLE	xmtr_specific_status_1 ;
	VARIABLE	xmtr_specific_status_2 ;
	VARIABLE	xmtr_specific_status_3 ;
	VARIABLE	xmtr_specific_status_4 ;
	VARIABLE	xmtr_specific_status_5 ;


	VARIABLE	response_preambles ;
	VARIABLE	burst_command_number ;
	VARIABLE	burst_mode_select ;

    COMMAND     write_pv_damping_value ;                /* Command  34 */
    COMMAND     reset_configuration_change_flag ;       /* Command  38 */
    COMMAND     enter_exit_fixed_pv_current_mode ;      /* Command  40 */
    COMMAND     perform_master_reset ;                  /* Command  42 */
	 COMMAND		 write_pv_units ;                        /* Command  44 */
	 COMMAND		 read_additional_transmitter_status ;    /* Command  48 */
	 COMMAND		 write_number_of_response_preambles ;    /* Command  59 */
	 COMMAND		 write_burst_command_number ;            /* Command 108 */
	 COMMAND		 burst_mode_control ;                    /* Command 109 */


    REDEFINITIONS {
	DELETE COMMAND read_dynamic_variables_and_pv_current;

        VARIABLE burst_command_number
        {
            REDEFINE TYPE ENUMERATED
            {
                {1, [primary_variable] } ,
                {2, [percent_range_and_current] } ,
                {3, [all_process_variables_and_current] }
            }
        }

        VARIABLE burst_mode_select
        {
            REDEFINE TYPE ENUMERATED
            {
                {1, [on]},
                {0, [off]}
            }
        }


			 VARIABLE xmtr_specific_status_0
			 {
			  REDEFINE TYPE BIT_ENUMERATED
			  {
			   { 0x01, ""   , IGNORE_IN_HANDHELD } ,
			   { 0x02, "|en|A111    electronics defect"
               "|de|A111    Elektronik defekt"
               "|fr|A111    defaut electronique"
               "|it|A111    Guasto elettron." , HARDWARE & UNCORRECTABLE } ,
			   { 0x04, "|en|A113    electronics defect"
               "|de|A113    Elektronik defekt"
               "|fr|A113    defaut electronique"
               "|it|A113    Guasto elettron." , HARDWARE & UNCORRECTABLE } ,
			   { 0x08, "|en|A114    electronics defect"
               "|de|A114    Elektronik defekt"
               "|fr|A114    defaut electronique"
               "|it|A114    Guasto elettron." , HARDWARE & UNCORRECTABLE } ,
			   { 0x10, "|en|A164    electronics defect"
               "|de|A164    Elektronik defekt"
               "|fr|A164    defaut electronique"
               "|it|A164    Guasto elettron." , HARDWARE & UNCORRECTABLE } ,
			   { 0x20, "|en|A102    checksum error - general reset & new calibr.required"
               "|de|A102    Pruefsummenfehler - Totalreset & Neuabgl. erfordert"
               "|fr|A102    erreur somme test - reset general nouvel etalonn."
               "|it|A102    errore check sum - ripristino totale nuova calibraz." , HARDWARE & UNCORRECTABLE } ,
			   { 0x40, "|en|A121    electronics defect"
               "|de|A121    Elektronik defekt"
               "|fr|A121    defaut electronique"
               "|it|A121    Guasto elettron." , HARDWARE & UNCORRECTABLE } ,
			   { 0x80, "|en|A171    electronics defect"
               "|de|A171    Elektronik defekt"
               "|fr|A171    defaut electronique"
               "|it|A171    Guasto elettron." , HARDWARE & UNCORRECTABLE }
			  }
			 }

			 VARIABLE xmtr_specific_status_1
			 {
		          REDEFINE TYPE BIT_ENUMERATED
		          {
			   { 0x01, "|en|A110    checksum error - general reset & new calibr.required"
               "|de|A110    Pruefsummenfehler - Totalreset & Neuabgl. erfordert"
               "|fr|A110    erreur somme test - reset general nouvel etalonn."
               "|it|A110    errore check sum - ripristino totale nuova calibraz.", HARDWARE & UNCORRECTABLE } ,
			   { 0x02, "|en|A160    checksum error - general reset & new calibr.required"
               "|de|A160    Pruefsummenfehler - Totalreset & Neuabgl. erfordert"
               "|fr|A160    erreur somme test - reset general nouvel etalonn."
               "|it|A160    errore check sum - ripristino totale nuova calibraz.", HARDWARE & UNCORRECTABLE } ,
			   { 0x04, "|en|A155    electronics defect"
               "|de|A155    Elektronik defekt"
               "|fr|A155    defaut electronique"
               "|it|A155    Guasto elettron.", HARDWARE & UNCORRECTABLE } ,
			   { 0x08, "|en|A231    sensor 1 defect - check connection"
               "|de|A231    Sensor 1 defekt - Pruefe Verbindung"
               "|fr|A231    defaut sonde 1 - verifier connexion"
               "|it|A231    Sens.1 guasto - controll. conness.", HARDWARE & UNCORRECTABLE } ,
			   { 0x10, "|en|A115    electronics defect"
               "|de|A115    Elektronik defekt"
               "|fr|A115    defaut electronique"
               "|it|A115    Guasto elettron.", HARDWARE & UNCORRECTABLE } ,
			   { 0x20, "|en|A272    electronics defect - amplifier"
               "|de|A272    Elektronik defekt - Verstaerker"
               "|fr|A272    defaut electron. - amplificateur"
               "|it|A272    guasto elettron. - amplificatore", HARDWARE & UNCORRECTABLE } ,
			   { 0x40, "|en|A106    downloading - please wait"
               "|de|A106    Download laeuft - bitte warten"
               "|fr|A106    download en cours - attendre"
               "|it|A106    download in corso - attendere", HARDWARE & UNCORRECTABLE } ,
			   { 0x80, "|en|A116    download error - repeat download"
               "|de|A116    Downloadfehler - Download wiederholen"
               "|fr|A116    erreur download - repeter download"
               "|it|A116    Errore download - ripetere download", HARDWARE & UNCORRECTABLE }
		          }
			 }

			 VARIABLE xmtr_specific_status_2
			 {
		          REDEFINE TYPE BIT_ENUMERATED
		          {
			   { 0x01, "|en|E641    no usable echo channel 1 - check calibr."
               "|de|E641    kein auswertbares Echo K1 - Abgleich pruefen"
               "|fr|E641    pas d'echo evaluable en voie 1 - verifier calib."
               "|it|E641    mancanza eco per elabor. C1 - controll. cali.", PROCESS & UNCORRECTABLE } ,
			   { 0x02, "|en|A671    linearisation ch1 not complete, not usable"
               "|de|A671    Linearisation Ch1 nicht vollstaendig, unbrauchbar"
               "|fr|A671    linearistion voie incomplete, nutilisable"
               "|it|A671    linearizzaz. C1 non completa, non utilizzabile", HARDWARE & UNCORRECTABLE } ,
			   { 0x04, "|en|E651    level in safety distance - risk of overspill"
               "|de|E651    Sicherheitsabst. erreicht - Ueberfuellgefahr"
               "|fr|E651    distance de securite atteinte - danger debord."
               "|it|E651    Raggiunta distanza di sicurezza - Pericolo tracim", PROCESS & UNCORRECTABLE } ,
			   { 0x08, "|en|A270    custody switch undef - check position"
               "|de|A270    Schalter undefiniert - ueberpruefe Position"
               "|fr|A270    commutateur indefini - verifier position"
               "|it|A270    Interr. non def. - contr. posizione", HARDWARE & UNCORRECTABLE } ,
			   { 0x10, "|en|E641    no usable echo channel 1 - check calibr."
               "|de|E641    kein auswertbares Echo K1 - Abgleich pruefen"
               "|fr|E641    pas d'echo evaluable en voie 1 - verifier calib."
               "|it|E641    mancanza eco per elabor. C1 - controll. cali.", PROCESS & CORRECTABLE } ,
			   { 0x20, "|en|E651    level in safety distance - risk of overspill"
               "|de|E651    Sicherheitsabst. erreicht - Ueberfuellgefahr"
               "|fr|E651    distance de securite atteinte - danger debord."
               "|it|E651    Raggiunta distanza di sicurezza - Pericolo tracim", PROCESS & CORRECTABLE } ,
			   { 0x40, "|en|W271    reduced accuracy - check installation"
               "|de|W271    reduz. Genauigkeit - Installation pruefen"
               "|fr|W271    precision reduit - verifier installat."
               "|it|W271    precis. ridotto - controll. Installaz.", STATE & SELF_CORRECTING } ,
			   { 0x80, "|en|W273    reduced accuracy"
               "|de|W273    reduz. Genauigkeit"
               "|fr|W273    precision reduit"
               "|it|W273    precis. ridotto", STATE & SELF_CORRECTING }
		          }
			 }

			 VARIABLE xmtr_specific_status_3
			 {
		          REDEFINE TYPE BIT_ENUMERATED
		          {
			   { 0x01, "|en|W274    reduced accuracy - check pipe diameter"
               "|de|W274    reduz. Genauigkeit - Pruefe Rohrdurchmesser"
               "|fr|W274    precision reduit - ver. diametre du tube"
               "|it|W274    precis. ridotto - controll. diametro del tubo guida", STATE & SELF_CORRECTING } ,
			   { 0x02, "|en|W275    electronics defect - factory settings"
               "|de|W275    Elektronik defekt - Werksabgleich"
               "|fr|W275    defaut electron. - calibration usine"
               "|it|W275    defaut electron. - calibrazione di fabbrica", STATE & SELF_CORRECTING } ,
			   { 0x04, "|en|W681    current ch1 out of range"
               "|de|W681    Strom Ch1 ausserhalb des Messbereichs"
               "|fr|W681    courant voie hors gamme"
               "|it|W681    usc. corr. C1 fuori range", STATE & SELF_CORRECTING } ,
			   { 0x08, "|en|W512    recording of mapping - please wait"
               "|de|W512    Aufnahme Ausblendung - warten"
               "|fr|W512    suppressionen cours - attendre"
               "|it|W512    registr. mappatura - in corso-attendere", STATE & SELF_CORRECTING } ,
			   { 0x10, "|en|W511    no factory calibration ch1"
               "|de|W511    kein Werksabgl. vorhanden"
               "|fr|W511    pas de reglage usine disponible voie 1"
               "|it|W511    assenza calibr. di fabbrica C1", STATE & SELF_CORRECTING } ,
			   { 0x20, "|en|W611    less than 2 linearisation points for channel 1"
               "|de|W611    Linearisierungspkt. Anzahl <2 (K1)"
               "|fr|W611    linearisation voie 1 moins de 2 points"
               "|it|W611    N punti linearizz.  minori di 2 (C1)", STATE & SELF_CORRECTING } ,
			   { 0x40, "|en|W601    linearisation ch1 - curve not monotone"
               "|de|W601    Linearisierung K1 - Kurve nicht monoton"
               "|fr|W601    linearisation voie 1 - courbe non monotone"
               "|it|W601    Linearizzazione C1 - curva non monotona", STATE & SELF_CORRECTING } ,
			   { 0x80, "|en|W621    simulation ch. 1 - on"
               "|de|W621    Simulation K1 - eingeschaltet"
               "|fr|W621    simulation voie 1 - active"
               "|it|W621    Simulazione C1 - attiva", STATE & SELF_CORRECTING }
		          }
			 }

			 VARIABLE xmtr_specific_status_4
			 {
		          REDEFINE TYPE BIT_ENUMERATED
		          {
			   { 0x01, "|en|W103    initialising - please wait"
               "|de|W103    Initialisierung - bitte warten"
               "|fr|W103    initialisation - en cours-attendre"
               "|it|W103    inizializzazione - in corso-attendere", STATE & SELF_CORRECTING } ,
			   { 0x02, "|en|W153    initialising - please wait"
               "|de|W153    Initialisierung - bitte warten"
               "|fr|W153    initialisation - en cours-attendre"
               "|it|W153    inizializzazione - in corso-attendere", STATE & SELF_CORRECTING } ,
			   { 0x04, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x08, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x10, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x20, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x40, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x80, ""  , IGNORE_IN_HANDHELD }
		          }
			 }

			 VARIABLE xmtr_specific_status_5
			 {
		          REDEFINE TYPE BIT_ENUMERATED
		          {
			   { 0x01, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x02, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x04, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x08, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x10, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x20, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x40, ""  , IGNORE_IN_HANDHELD } ,
			   { 0x80, ""  , IGNORE_IN_HANDHELD }
		          }
			 }



        COMMAND read_additional_transmitter_status
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                }
                REPLY
                {
                     response_code,
                     device_status,
                     xmtr_specific_status_0,
		               xmtr_specific_status_1,
		               xmtr_specific_status_2,
		               xmtr_specific_status_3,
		               xmtr_specific_status_4,
		               xmtr_specific_status_5
                }
            }
        }

    }
}





// Parameter: YT1_VALUE, YT1_URV, YT1_LRV, YT1_LABEL
#define YT1_VALUE       parMeasuredValue
#define YT1_URV         MEASVAR_UPPER_LIMIT
#define YT1_LRV         MEASVAR_LOWER_LIMIT
#define YT1_LABEL       "|en|measured value|de|Messwert|fr|Valeur de mesure|es|Messwert|it|Valore di misura|nl|gemeten waarde"

/*** Includes ********************************************************************/

#include "\Hart\Hcf\bi_codes.h"
#include "\Hart\Hcf\macros.ddl"
// #include "\Hart\SIHA.inc"
//#include "MPM_HA.inc"      // (JM) Copied and modified from SIHA.inc
                           // This checks that the device release is
                           // CHECK_SOFTWARE_VERSION or CHECK_SOFTWARE_VERSION02
			

#include "\SI_FileM.inc"
#include "\SI_HlpM.inc"
#include "\SI_OptM.inc"
#include "\SI_DevM.inc"
#include "\SI_ViewM.inc"
#include "\SI_YT.inc"
#define  DEFAULT_MANUFACTURER_ID        17
/* !Defines! ***************************************************************************/
#define  Val006	((parOperationCode == 33999)||\
			(parOperationCode ==33998)||\
			(parOperationCode == 200)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))
#define  Val016	(parSoftwareVersion > 1)
#define  Val017	((Val006) && (parSoftwareVersion == 1))
#define  Dlc000	Val016
#define  Hnd000	(parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123)
#define  Hnd001	((parOperationCode == 100)||\
			 (parOperationCode == 300)||\
			 (parOperationCode == 123))\
			 &&\
			((parLevelUllageMode == 0)||\
			 (parLevelUllageMode == 2))
#define  Hnd002	parOperationMode != 1
#define  Hnd003	(parOperationCode == 300)||\
			(parOperationCode == 123)
#define  Hnd004	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))\
			&&\
			(parLinearisation == 241)&&\
			((parLevelUllageMode == 0)||\
			(parLevelUllageMode == 2))
#define  Hnd005	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))\
			&&\
			((parLinearisation == 242)||\
			(parLinearisation == 241))\
			&&\
			((parLevelUllageMode == 0)||\
			(parLevelUllageMode == 2))
#define  Hnd006	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))\
			&&\
			((parLinearisation == 0)||\
			(parLinearisation == 245))\
			&&\
			((parLevelUllageMode == 0)||\
			(parLevelUllageMode == 2))
#define  Hnd007	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))\
			&&\
			(parLinearisation == 245)\
			&&\
			((parLevelUllageMode == 0)||\
			(parLevelUllageMode == 2))
#define  Hnd008	(parOperationCode == 123)
#define  Hnd009	((parOperationCode == 100)||\
			(parOperationCode == 200)||\
			(parOperationCode == 300)||\
			(parOperationCode == 33998)||\
			(parOperationCode == 123))
#define  PostEditCond_1	((preDistanceUnit == 44 && parDistanceUnit == 45) ||\
			(preDistanceUnit == 44 && parDistanceUnit == 49) ||\
			(preDistanceUnit == 47 && parDistanceUnit == 45) ||\
			(preDistanceUnit == 47 && parDistanceUnit == 49))
#define  PostEditCond_2	((preDistanceUnit == 45 && parDistanceUnit == 44) ||\
			(preDistanceUnit == 45 && parDistanceUnit == 47) ||\
			(preDistanceUnit == 49 && parDistanceUnit == 44) ||\
			(preDistanceUnit == 49 && parDistanceUnit == 47))
#define  Hnd010	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123)\
			) &&\
			(parCheckDistance == 241)
#define  Val000	(((parOperationCode==100)||(parOperationCode==300)||(parOperationCode==123))&&((parLinearisation==242)||(parLinearisation==241))&&((parLevelUllageMode==0)||(parLevelUllageMode==2)))
#define  Val001	(parLinearisation==242)
#define  Val002	(parLinearisation==241)
#define  Val003	((parLinearisation==245)||(parLinearisation==0))
#define  Val004	(parLinearisation==245)
#define  Val005	(parFormatDisplay_ft==0)
#define  Val007	((parTankShape==3)||(parTankShape==2))
#define  Val008	(parOutputOnAlarm==253)
#define  Val009	(parReactionLostEcho==2)
#define  Val010	((parCheckDistance==0)||(parCheckDistance==1)||(parCheckDistance==241))
#define  Val011	(parFixedCurrentOutput==1)
#define  Val012	(parSimulation==1)
#define  Val013	(parSimulation==3)
#define  Val014	(parSimulation==2)
#define  Hnd011	((parOperationCode == 33999)||\
			(parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 123))
#define  Val015	((parLevelUllageMode == 0)||(parLevelUllageMode == 2))\
	&& ((parLinearisation ==0)||(parLinearisation == 245))
#define  Hnd012	(parOperationCode != 33998)
#define  Hnd013	((parOperationCode == 100)||\
			(parOperationCode == 300)||\
			(parOperationCode == 33998)||\
			(parOperationCode == 123))
#define  Dlc001	Val017
#define  Val018	(parSoftwareVersion == 1)
#define  Dlc002	Val018

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  percent_range {
	LABEL 	"PERCENT RANGE" ;
	CLASS 	INPUT ; 
	HANDLING  READ ;
	CONSTANT_UNIT  "%" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"6.1f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  output_transfer_function {
	LABEL 	[transfer_function]  ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE 	ENUMERATED	{
         TRANSFER_FUNCTION_CODE (0) ,
         { 245     , "|en|horizonzal zyl.		|de|zylindrisch liegend	        |fr|	|it|				" },
         { 241  , "|en|manual			|de|manuell			|fr|	|it|				" },
         { 242    , "|en|semi automatic		|de|halbautomatisch		|fr|    |it|				" },
         { 4  , "|en|activate table		|de|Tabelle aktivieren		|fr|	|it|				" },
         { 243   , "|en|clear table		|de|Tabelle loeschen		|fr|    |it|				" }
    }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  sensor_minimum_span {
	LABEL 	"MIN. SPAN"  ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"6.1f"  ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  sensor_lower_limit {
	LABEL 	"LOW SENSOR LIMIT" ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	".4f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  sensor_upper_limit {
	LABEL 	"HIGH SENSOR LIMIT"  ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	".4f"  ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  sensor_serial_number {
	LABEL 	"SERIAL-NO. SENS."  ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE  UNSIGNED_INTEGER(3)  {
		DISPLAY_FORMAT	"7u" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  format {
	LABEL 	"FORMAT"  ;
	CLASS 	CORRECTION  ; 
	HANDLING  READ  ;
	TYPE  UNSIGNED_INTEGER  {
		DISPLAY_FORMAT	"01u"  ;
		MIN_VALUE 	0  ;
		MAX_VALUE	4  ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  nul_unit {
	LABEL 	"NUL_UNIT"  ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE 	ENUMERATED	{
        { 251, "none" }
    }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  dummy_float {
	LABEL 	"DUMMY_FLOAT" ;
	CLASS 	INPUT ; 
	HANDLING  READ  ;
	TYPE  FLOAT ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMeasuredValue {
	LABEL 	"|en|measured value|de|Messwert|fr|Valeur de mesure|es|Messwert|it|Valore di misura|nl|gemeten waarde" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	TYPE 	FLOAT	{
        DISPLAY_FORMAT

        IF (parNoOfDecimals == 0)
        {
            "9.0f";
        }
        ELSE
        {
            IF ( parNoOfDecimals == 1 )
            {
                "9.1f";
            }
            ELSE
            {
                IF ( parNoOfDecimals == 2 )
                {
                    "9.2f";
                }
                ELSE
                {
                    IF ( parNoOfDecimals == 3 )
                    {
                        "9.3f";
                    }
                        ELSE
                       {
                        "9.3f";
                       }
                    }
                }
            }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parNoOfDecimals {
	LABEL 	"|en|no.of decimals|de|Nachkommast.|fr|decimales|es|Nachkommast.|it|decimali|nl|aantal decim." ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val005)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   2; 
	{ 0 ,"|en|x"}, 
	{ 1 ,"|en|x.x"}, 
	{ 2 ,"|en|x.xx"}, 
	{ 3 ,"|en|x.xxx"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parTankShape {
	LABEL 	"|en|tank shape|de|Tankgeometrie|fr|Forme reservoir|es|Tankgeometrie|it|Forma serbatoio|nl|Tankvorm" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  { 0 ,"|en|dome ceiling|de|Kloepperdeckel|fr|Toit bombe|es|techo bveda|it|tetto bombato|nl|dome plafond"}, { 245 ,"|en|horizontal cyl|de|zyl.liegend|fr|cyl. horizont.|es|cilind. horiz.|it|cilind. orizz.|nl|horizont. cyl."}, { 2 ,"|en|bypass|de|Bypass|fr|Bypass|es|bypass|it|bypass|nl|by-pass"}, { 3 ,"|en|stilling well|de|Schwallrohr|fr|Tube de mesure|es|tubo tranquil.|it|tubo di calma|nl|duikpyp"}, { 4 ,"|en|flat ceiling|de|Flachdeckel|fr|Toit plat|es|techo plano|it|tetto piatto|nl|vlak plafond"}, { 5 ,"|en|sphere|de|Kugeltank|fr|Spherique|es|esfera|it|sfera|nl|sferisch"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOperationCode {
	LABEL 	"|en|unlock parameter|de|Freigabecode|fr|code operation|es|Freigabecode|it|Modo operativo|nl|startcode" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd002)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"5.1u" ;
		EDIT_FORMAT	"5.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	339999 ;
		DEFAULT_VALUE	100 ;
		INITIAL_VALUE	100 ;
	}
	POST_READ_ACTIONS  { 
		updateParShowAccess
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDielectricConstant {
	LABEL 	"|en|medium property|de|Medium Eigensch.|fr|charact.du prod|es|Medium Eigensch.|it|caratt.del prod.|nl|produktconstante" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  { 0 ,"|en|unknown|de|unbekannt|fr|inconnu|es|desconocido|it|sconosciuta|nl|onbekend"}, { 1 ,"|en|DC:< 1.9"}, { 2 ,"|en|DC:1.9 ... 4"}, { 3 ,"|en|DC:4 ... 10"}, { 4 ,"|en|DC:> 10"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parProcessCondition {
	LABEL 	"|en|process cond.|de|Messbedingungen|fr|Cond. process|es|Messbedingungen|it|condiz. Processo|nl|proces conditier" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  
	{ 0 ,"|en|standard|de|Standard|fr|standard|es|estndar|it|standard|nl|standaard"}, 
	{ 1 ,"|en|calm surface|de|Oberfl. ruhig|fr|Surface calme|es|sup. calmada|it|superf.calma|nl|rustig"}, 
	{ 2 ,"|en|turb. surface|de|Oberfl.unruhig|fr|Surface agitee|es|sup. turbul.|it|superf.turbol.|nl|onrustig"}, 
	{ 3 ,"|en|add. agitator|de|zus. Ruehrwerk|fr|supp.Agitateur|es|adic. agitador|it|supp.agitatore|nl|plus roerwerk"}, 
	{ 4 ,"|en|fast change|de|schnelle aender|fr|Variat. rapide|es|camb. rpidos|it|cambiam.veloci|nl|snelle variat."}, 
	{ 5 ,"|en|test:no filter|de|Test:Filt. aus|fr|Test:pas filt.|es|test:no filtro|it|test:no filtro|nl|test:no filter"}  
}
	POST_EDIT_ACTIONS  { 
		synchronizeParProcessConditionToParOutputDamping
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEmptyCalibration {
	LABEL 	"|en|empty calibr.|de|Abgleich leer|fr|Etalonnage vide|es|Abgleich leer|it|calibr.di vuoto|nl|leegafregeling" ;
	HELP 	"|en|distance processconn. to min. level|de|Abstand Flansch zu min.Fuellstand|fr|Distance brideau niveau min.|es|dimetro internocon, al nivel mnimo|it|distanza min livellodalla flangia|nl|afstand van de flenstot het min. niveau" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	        {
                                        EDIT_FORMAT "9.3f";
                                        DISPLAY_FORMAT "9.3f";
                                        MIN_VALUE 0.0;
                                        MAX_VALUE
                                        IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
                                        }
                                        ELSE {
                                                SELECT (parDistanceUnit)
                                                {
                                                        CASE 45: 22.0;
                                                        CASE 44: 72.178;
                                                        CASE 47: 866.142;
                                                        CASE 49: 22000.0;
                                                        DEFAULT: 22.0;
                                                }
                                        }
                                        DEFAULT_VALUE
                                        IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
                                        }
                                        ELSE {
                                                SELECT (parDistanceUnit)
                                                {
                                                        CASE 45: 22.0;
                                                        CASE 44: 72.178;
                                                        CASE 47: 866.142;
                                                        CASE 49: 22000.0;
                                                        DEFAULT: 22.0;
                                                }
                                        }
                                }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAbsMaxMessDistance {
	LABEL 	"|en|max. meas. dist." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
		DEFAULT_VALUE	22.0 ;
	}
	POST_READ_ACTIONS  { 
		setAbsMax
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDistanceUnit {
	LABEL 	"|en|distance unit|de|Laengeneinheit|fr|unite longueur|es|Laengeneinheit|it|Unit distanza|nl|lengte-eenheid" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   45;
	{ 45 ,"|en|m"}, 
	{ 44 ,"|en|ft"}, 
	{ 49 ,"|en|mm"}, 
	{ 47 ,"|en|inch"}  
}
	POST_EDIT_ACTIONS  { 
		IF (PostEditCond_1) { nonMetricToMetricUnits } IF (PostEditCond_2) { metricToNonMetricUnits } assignDistanceUnit
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFullCalibration {
	LABEL 	"|en|full calibr.|de|Abgleich voll|fr|Etalonnage plein|es|Abgleich voll|it|calibr.di pieno|nl|volafregeling" ;
	HELP 	"|en|span|de|Messspanne|fr|Gamme de mesure|es|campo de medida|it|campo di misura|nl|meetbereik" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
						parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit) {
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE
					IF (parMaxAbs) {
						parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit) {
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOperationMode {
	LABEL 	"|en|overspill prot.|de|berfllsicherung|fr|anti-debordement|es|ueberfuellsicher.|it|prot. antiracim|nl|overvulbewaking" ;
	CLASS 	INPUT ; 
	HANDLING  READ & WRITE ;
	TYPE 	ENUMERATED	{  { 0 ,"|en|standard|de|Standard|fr|standard|es|estndar|it|standard|nl|standaard"}, { 1 ,"|en|german WHG|de|WHG|fr|WHG|es|alemana WHG|it|WHG tedesco|nl|duitse WHG"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFormatDisplay_ft {
	LABEL 	"|en|format display|de|Anzeigeformat|fr|format affichage|es|Anzeigeformat|it|formato display|nl|aanwijsformaat" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   0;
	{ 0 ,"|en|decimal|de|dezimal|fr|decimal|es|decimal|it|decimale|nl|decimaal"}, 
	{ 1 ,"|en|1/16''"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOutputOnAlarm {
	LABEL 	"|en|output on alarm|de|Ausg. b. Alarm|fr|Sortie si alarme|es|Ausg. b. Alarm|it|Comp. in allarme|nl|uitg. bij alarm" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   0;
	{ 1 ,"|en|MIN -10% 3.6mA|de|MIN -10% 3.6mA|fr|MIN -10% 3,6mA|es|MIN -10% 3.6mA|it|MIN -10% 3.6mA|nl|MIN -10% 3.6mA"}, 
	{ 0 ,"|en|MAX 110% 22mA|de|MAX 110% 22mA|fr|MAX 110% 22mA|es|MAX 110% 22mA|it|MAX 110% 22mA|nl|MAX 110% 22mA"}, 
	{ 239 ,"|en|hold|de|Halten|fr|Maintien|es|Mantener|it|Ultimo valore|nl|Hold"}, 
	{ 253 ,"|en|user specific|de|anwenderspez.|fr|vue specifique|es|espec. usuario|it|spec. cliente|nl|gebr.specifiek"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parReactionLostEcho {
	LABEL 	"|en|outp. echo loss|de|Ausg.Echoverlust|fr|Sort. perte echo|es|Ausg.Echoverlust|it|in caso manc.eco|nl|uitg.echoverlies" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   239; 
	{ 0 ,"|en|alarm|de|Alarm|fr|Alarme|es|alarma|it|allarme|nl|Alarm"}, 
	{ 239 ,"|en|hold|de|Halten|fr|Maintien|es|Mantener|it|Ultimo valore|nl|Hold"}, 
	{ 2 ,"|en|ramp %/min|de|Rampe %/min|fr|Rampe %/min|es|rampa %/min|it|rampa %/min|nl|Steilh. %/min"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parTubeDiameter {
	LABEL 	"|en|pipe diameter|de|Rohrdurchmesser|fr|Diametre du tube|es|Rohrdurchmesser|it|diametro tubo|nl|diam. by-pass" ;
	HELP 	"|en|inner diameter ofbypass/stilling well|de|InnendurchmesserBypass/Schwallrohr|fr|Diametre interieurdu tube/by-pass|es|dimetro internobypass/tubo tranq.|it|diametro internodel tubo guida|nl|inwendige diam. vanby-pass / duikpijp" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val007)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
                   EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :    1.000;
						CASE 44 :    0.039;
						CASE 47 :    0.039;
						CASE 49 :    1.000;
						DEFAULT:       1.000;
					};
					MAX_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :  10000.000;
						CASE 44 :    393.701;
						CASE 47 :    393.701;
						CASE 49 :  10000.000;
						DEFAULT:     10000.000;
					};
					DEFAULT_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :    204.425;
						CASE 44 :      8.048;
						CASE 47 :      8.048;
						CASE 49 :    204.425;
						DEFAULT:       204.425;
					};
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOutputAtError {
	LABEL 	"|en|output on alarm|de|Ausg. b. Alarm|fr|Sortie si alarme|es|Ausg. b. Alarm|it|Comp.in allarme|nl|uitg. bij alarm" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val008)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|mA" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"4.2f" ;
		EDIT_FORMAT	"4.2f" ;
		MIN_VALUE 	3.6 ;
		MAX_VALUE	22.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parRampInPercentPerMin {
	LABEL 	"|en|ramp %span/min|de|Rampe %MB/min|fr|Rampe %GM/min|es|Rampe %MB/min|it|rampa %campo/min|nl|steilh.span%/min" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val009)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|%/min" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"10.3f" ;
		EDIT_FORMAT	"10.3f" ;
		MIN_VALUE 	-99999.0 ;
		MAX_VALUE	99999.0 ;
		DEFAULT_VALUE	0.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLevelUllageMode {
	LABEL 	"|en|level/ullage|de|Fuellst./Restvol.|fr|Niv./Vol. resid.|es|Fuellst./Restvol.|it|livello/vol vuot|nl|niveau/restvol." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   0; 
	{ 0 ,"|en|level CU|de|Fuellst. TE|fr|niveau  UT|es|nivel unid us.|it|livello UI|nl|niveau T.E.."}, 
	{ 1 ,"|en|level DU|de|Fuellst m/ft/in|fr|niveau m/ft/in|es|nivel m/p/pulg|it|livel. m/ft/in|nl|niveau m/ft/in"}, 
	{ 2 ,"|en|ullage CU|de|Restvol. TE|fr|vol. resid. UT|es|unidades usu.|it|vol vuoto UI|nl|restvol. T.E."}, 
	{ 3 ,"|en|ullage DU|de|Restvo.m/ft/in|fr|vo.res.m/ft/in|es|unida m/p/pulg|it|vol vu.m/ft/in|nl|restv. m/ft/in"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLinearisation {
	LABEL 	"|en|linearisation|de|Linearisierung|fr|Linearisation|es|Linearisierung|it|Linearizzazione|nl|linearisatie" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd001)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE   0; 
	{ 0 ,"|en|linear|de|linear|fr|lineaire|es|lineal|it|Lineare|nl|lineair"}, 
	{ 245 ,"|en|horizontal cyl|de|zyl.liegend|fr|cyl. horizont.|es|cilind. horiz.|it|cilind. orizz.|nl|horizont. cyl."}, 
	{ 241 ,"|en|manual|de|manuell|fr|manuel|es|manual|it|manuale|nl|handmatig"}, 
	{ 242 ,"|en|semi-automatic|de|halbautomat.|fr|semi-automat.|es|semiautomatico|it|semi-auto.|nl|half-automat."}, 
	{ 4 ,"|en|table on|de|Tabelle ein|fr|tableau actif|es|tabla on|it|tab. attivata|nl|tabel aan"}, 
	{ 243 ,"|en|clear table|de|Loesche Tabelle|fr|effac. tableau|es|borrar tabla|it|cancella tab.|nl|tabel wissen"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDelayTimeOnLostEcho {
	LABEL 	"|en|delay time|de|Verzoegerung|fr|Temporisation|es|Verzoegerung|it|Tempo di ritardo|nl|vertragingstijd" ;
	HELP 	"|en|in case of echo lossmax. 4000 sec.|de|Zeit bei Echoverlustmax. 4000 sec.|fr|si perte echomax. 4000 sec.|es|en caso de perd. ecomx. 4000 segundos|it|se manca ecomax. 4000 sec.|nl|i.g.v. echoverliesmax. 4000 s." ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|s" ;
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"4.1u" ;
		EDIT_FORMAT	"4.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	4000 ;
		DEFAULT_VALUE	30 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLevelWithinSafetyDistance {
	LABEL 	"|en|safety distance|de|Sicherheitsabst.|fr|Dist. securite|es|Sicherheitsabst.|it|dist. di sicur.|nl|veiligheidsafst." ;
	HELP 	"|en|from antenna tip/lower edge of horn|de|ab Antennenspitze/Hornkante|fr|sous antenne/extremite antenne|es|desde extremo antenadesde extr. trompeta|it|dall'antenna/bordo inf. antenna|nl|vanaf antennetip/onderkant hoornant." ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
						parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :  0.100;
						CASE 44 :  0.328;
						CASE 47 :  3.937;
						CASE 49 :  100.000;
						DEFAULT:  0.100;
					};
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parInSafetyDistance {
	LABEL 	"|en|in safety dist.|de|im Sicherh.abst.|fr|dans dist.secur.|es|im Sicherh.abst.|it|in dist.di sicur|nl|op veiligh.afst." ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE   1; { 0 ,"|en|alarm|de|Alarm|fr|Alarme|es|alarma|it|allarme|nl|Alarm"}, { 1 ,"|en|warning|de|Warnung|fr|Avertissement|es|aviso|it|avviso|nl|Waarschuwing"}, { 2 ,"|en|self holding|de|Selbsthaltung|fr|Maintien|es|mantener|it|autoritenuto|nl|Hold"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAcknowledgeAlarm {
	LABEL 	"|en|ackn. alarm|de|Reset Selbsthalt|fr|Supp.auto maint.|es|Reset Selbsthalt|it|ripristina|nl|reset alarm" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd013)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE   0; { 0 ,"|en|no|de|nein|fr|non|es|no|it|no|nl|nee"}, { 1 ,"|en|yes|de|ja|fr|oui|es|si|it|si|nl|ja"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCustomerUnit {
	LABEL 	"|en|customer unit|de|Kundeneinheit|fr|Unite utilisat.|es|Kundeneinheit|it|Unit di misura|nl|klanteenheid" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd001)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 57; 
	{ 57 ,"|en|%"}, 
	{ 41 ,"|en|l"}, 
	{ 236 ,"|en|hl"}, 
	{ 43 ,"|en|m"}, 
	{ 244 ,"|en|dm"}, 
	{ 245 ,"|en|cm"}, 
	{ 112 ,"|en|ft"}, 
	{ 40 ,"|en|us_gal"}, 
	{ 42 ,"|en|i_gal"}, 
	{ 61 ,"|en|kg"}, 
	{ 62 ,"|en|t"}, 
	{ 63 ,"|en|lb"}, 
	{ 64 ,"|en|ton"}, 
	{ 45 ,"|en|m"}, 
	{ 44 ,"|en|ft"}, 
	{ 49 ,"|en|mm"}, 
	{ 47 ,"|en|inch"}, 
	{ 251 ,"|en|"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parTableNumber {
	LABEL 	"|en|table no.|de|Tabellen Nummer|fr|numero tableau|es|Tabellen Nummer|it|N tabella|nl|tabel nummer" ;
	HELP 	"|en|volume xx custunitlevel xx m/ft|de|Volumen xx Kd.EinFuellstand xx m/ft|fr|Volume xx u.util.Niveau xx m/ft|es|volum xx unid usunivel xx m/pies|it|Volume xx un.ing.Livello xx un.ing.|nl|vol.klanteenh.niveau xx m/ft" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val000)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd005)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"2.1u" ;
		EDIT_FORMAT	"2.1u" ;
		MIN_VALUE 	1 ;
		MAX_VALUE	32 ;
		DEFAULT_VALUE	1 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parInputLevelHalfAutomatic {
	LABEL 	"|en|input level|de|Eingabe Fuellst.|fr|entree niveau|es|Eingabe Fuellst.|it|Dato livello|nl|invoer niveau" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val001)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parInputLevelManual {
	LABEL 	"|en|input level|de|Eingabe Fuellst.|fr|entree niveau|es|Eingabe Fuellst.|it|Dato livello|nl|invoer niveau" ;
	HELP 	"|en|volume xx custunittable no. X|de|Volumen xx Kd.EinTabellennummer x|fr|Volume xx u.util.numero tableau x|es|vol xx unid,usunmero en la tabla x|it|Volume xx un.ing.Tabella n x|nl|vol.klanteenh.tabelnummer" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val002)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd004)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parInputVolume {
	LABEL 	"|en|input volume|de|Eingabe Volumen|fr|entree volume|es|Eingabe Volumen|it|Dato volume|nl|invoer volume" ;
	HELP 	"|en|level xx m/fttable no. X|de|Fuellstand xx m/ftTabellennummer x|fr|Niveau xx m/ftnumero tableau x|es|nivel xx en m/pnmero en la tabla x|it|Livello xx m/ftTabella n X|nl|niveau xx m/fttabelnummer X" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val000)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd005)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"10.3f" ;
		EDIT_FORMAT	"10.3f" ;
		MIN_VALUE 	-99999.0 ;
		MAX_VALUE	99999.0 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMaxVolume {
	LABEL 	"|en|max. scale|de|Endwert Messber.|fr|Gamme mes. max.|es|Endwert Messber.|it|max.valore|nl|eindw.meetbereik" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val003)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd006)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"10.3f" ;
		EDIT_FORMAT	"10.3f" ;
		MIN_VALUE 	-99999.0 ;
		MAX_VALUE	99999.0 ;
		DEFAULT_VALUE	100.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCylinderVessel {
	LABEL 	"|en|diameter vessel|de|Zyl.-durchmesser|fr|diametre cuve|es|Zyl.-durchmesser|it|Diam. Serbatoio|nl|tankdiameter" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val004)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd007)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "m";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :  9.000;
						CASE 44 :  29.528;
						CASE 47 :  354.331;
						CASE 49 :  9000.000;
						DEFAULT:  9.000;
					};
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCheckDistance {
	LABEL 	"|en|check distance|de|Distanz pruefen|fr|verifier dist.|es|Distanz pruefen|it|Controll. dist.|nl|check afstand" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 3; 
	{ 0 ,"|en|distance = ok|de|Distanz = ok|fr|distance = ok|es|distancia = ok|it|distanza = ok|nl|afstand = ok"}, 
	{ 1 ,"|en|dist.too small|de|Dist. zu klein|fr|dis.trop petit|es|dist. muy pequ|it|dist.trop.picc|nl|afst. te klein"}, 
	{ 2 ,"|en|dist. too big|de|Dist. zu gross|fr|dis.trop grand|es|dist.muy grand|it|dist.trop.gran|nl|afst. te groot"}, 
	{ 3 ,"|en|dist. unknown|de|Dist.unbekannt|fr|dist. inconnue|es|dist.desconoc.|it|dist. sconosc.|nl|afst.niet  bek"}, 
	{ 241 ,"|en|manual|de|manuell|fr|manuel|es|manual|it|manuale|nl|handmatig"}  
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSuppressionDistance {
	LABEL 	"|en|range of mapping|de|Bereich Ausblend|fr|Plage suppress.|es|Bereich Ausblend|it|dist. mappa man.|nl|bereik uitwaard." ;
	HELP 	"|en|input ofmapping range|de|Eingabe desAusbl.bereiches|fr|entree plagede suppression|es|introducirrango mapeado|it|dato distanzafino a mappa|nl|invoer van hetuitwaard. bereik" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val010)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd010)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parStartMappingRecord {
	LABEL 	"|en|start mapping|de|Starte Ausblend.|fr|Lancer mapping|es|Starte Ausblend.|it|avvico mappatura|nl|start mapping" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val010)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parPresMapRange {
	LABEL 	"|en|pres. map dist.|de|akt. Ausbl.dist.|fr|dist. suppr.act.|es|akt. Ausbl.dist.|it|dist. mappa att.|nl|act. afst. uitw." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCustTankMap {
	LABEL 	"|en|cust. tank map|de|Ausblendung|fr|suppression|es|Ausblendung|it|Mappa cliente|nl|klant tank map" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  { 0 ,"|en|not active"}, { 1 ,"|en|active"}, { 2 ,"|en|reset|de|loeschen|fr|reset|es|borrar|it|reset|nl|reset"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEchoQuality {
	LABEL 	"|en|echo quality|de|Echoqualitaet|fr|Qualite echo|es|Echoqualitaet|it|Qualit eco|nl|echokwaliteit" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOffsetOfMeasuredDistance {
	LABEL 	"|en|offset|de|Fuellhoehenkorrekt|fr|Correct. Niveau|es|Fuellhoehenkorrekt|it|offset|nl|offset" ;
	HELP 	"|en|will be added to themeasured level|de|wird zum gemessennenFuellstand addiert|fr|sera additionneau niveau mesure|es|sera aadidoal luivel medido|it|sara addizionataal livello misurato|nl|zal werd. toegevoegdan de gemeten waarde" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "10.3f";
					DISPLAY_FORMAT "10.3f";
					MIN_VALUE
					IF (parMaxAbs) {
                                                (-1.0)*parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: -22.0;
							CASE 44: -72.178;
							CASE 47: -866.142;
							CASE 49: -22000.0;
							DEFAULT: -22.0;
						}
					}
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOutputDamping {
	LABEL 	"|en|output damping|de|Integrationszeit|fr|temps integrat.|es|Integrationszeit|it|Tempo integraz.|nl|uitgangsdemping" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|s" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.1f" ;
		EDIT_FORMAT	"5.1f" ;
		MIN_VALUE 	0.0 ;
		MAX_VALUE	255.0 ;
		DEFAULT_VALUE	3.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parBlockingDistance {
	LABEL 	"|en|blocking dist.|de|Blockdistanz|fr|distance blocage|es|Blockdistanz|it|Dist. di blocco|nl|blokafstand" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
					DEFAULT_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :  0.500;
						CASE 44 :  1.640;
						CASE 47 :  19.685;
						CASE 49 :  500.000;
						DEFAULT:  0.500;
					};

				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCommAddress {
	LABEL 	"|en|commun. address|de|Kommun.Adresse|fr|adresse commun.|es|Kommun.Adresse|it|Indirizzo strum.|nl|communicat.adres" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"2.1i" ;
		EDIT_FORMAT	"2.1i" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	15 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parPreambles {
	LABEL 	"|en|no. of preambels|de|Praeambelanzahl|fr|Nb preambules|es|Praeambelanzahl|it|Introduzione|nl|aantal. inleid." ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"2.1i" ;
		EDIT_FORMAT	"2.1i" ;
		MIN_VALUE 	4 ;
		MAX_VALUE	20 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parThresholdMainValue {
	LABEL 	"|en|low output limit|de|Grenze Messwert|fr|Limite val. mes.|es|Grenze Messwert|it|Limite massimo|nl|grensw. uitgang" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ 
	DEFAULT_VALUE 1;
 { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFixedCurrentOutput {
	LABEL 	"|en|fixed current|de|fester Strom|fr|courant fixe|es|fester Strom|it|Corrente fissa|nl|vaste stroom" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val018)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFixedCurrentValue {
	LABEL 	"|en|fixed cur. value|de|fester Strom|fr|courant fixe|es|fester Strom|it|Corrente fissa|nl|vaste stroom" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val011)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|mA" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.2f" ;
		EDIT_FORMAT	"5.2f" ;
		MIN_VALUE 	3.8 ;
		MAX_VALUE	20.5 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSimulation {
	LABEL 	"|en|simulation|de|Simulation|fr|simulation|es|Simulation|it|simulazione|nl|simulatie" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|sim. off|de|Sim. aus|fr|simul. inact.|es|sim, conectada|it|sim. disattiva|nl|sim. off"}, { 1 ,"|en|sim. level|de|Sim. Fuellstand|fr|simul. niveau|es|sim, nivel|it|sim. livello|nl|sim. level"}, { 2 ,"|en|sim. volume|de|Sim. Volumen|fr|simul. volume|es|sim, volumen|it|sim. volume|nl|sim. volume"}, { 3 ,"|en|sim. current|de|Sim. Strom|fr|simul. courant|es|sim, corriente|it|sim. corrente|nl|sim. current"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSimulationValueLevel {
	LABEL 	"|en|simulation value|de|Simulationswert|fr|valeur simulat.|es|Simulationswert|it|Val. di simul.|nl|simulatiewaarde" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val012)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45 :  -2.000;
						CASE 44 :  -6.562;
						CASE 47 :  -78.740;
						CASE 49 :  -2000.000;
						DEFAULT:  -2000.000;
					};
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22.0;
							CASE 44: 72.178;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22.0;
						}
					}
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSimulationValueCurrent {
	LABEL 	"|en|simulation value|de|Simulationswert|fr|valeur simulat.|es|Simulationswert|it|Val. di simul.|nl|simulatiewaarde" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val013)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|mA" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.2f" ;
		EDIT_FORMAT	"5.2f" ;
		MAX_VALUE	22.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCurrentOutput {
	LABEL 	"|en|output current|de|Ausgangsstrom|fr|Sortie courant|es|Ausgangsstrom|it|uscita corrente|nl|uitgangsstroom" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|mA" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.2f" ;
		EDIT_FORMAT	"5.2f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLanguage {
	LABEL 	"|en|language|de|Sprache|fr|Langage|es|Sprache|it|Lingua|nl|taal" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd009)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|English"}, { 1 ,"|en|Deutsch"}, { 2 ,"|en|Franais"}, { 4 ,"|en|Italiano"}, { 3 ,"|en|Espaol"}, { 5 ,"|en|Nederlands"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parBackToHome {
	LABEL 	"|en|back to home|de|Zur Startseite|fr|Retour  Home|es|Zur Startseite|it|Vai a men princ|nl|Home" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|s" ;
	TYPE  INTEGER (2)  {
		DISPLAY_FORMAT	"4.1i" ;
		EDIT_FORMAT	"4.1i" ;
		MIN_VALUE 	3 ;
		MAX_VALUE	9999 ;
		DEFAULT_VALUE	100 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSeparationCharacter {
	LABEL 	"|en|sep. character|de|Trennungszeichen|fr|Signe separation|es|Trennungszeichen|it|segni separaz.|nl|scheidingsteken" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val005)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|."}, { 1 ,"|en|,"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCurrentAmplitude {
	LABEL 	"|en|pres. amplitude" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"3.1i" ;
		EDIT_FORMAT	"3.1i" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAmplitudeOverSuppression {
	LABEL 	"|en|ampl. over map." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"3.1i" ;
		EDIT_FORMAT	"3.1i" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAmplitudeOverFAC {
	LABEL 	"|en|ampl. over FAC" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"4.1i" ;
		EDIT_FORMAT	"4.1i" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parUnfilteredDistance {
	LABEL 	"|en|unfilt. distance" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"8.2f" ;
		EDIT_FORMAT	"8.2f" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCalcEdgeParameter {
	LABEL 	"|en|pres. edge param" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"4.1u" ;
		EDIT_FORMAT	"4.1u" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCurrentFirstEchoFactor {
	LABEL 	"|en|present FEF" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.2f" ;
		EDIT_FORMAT	"5.2f" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDeviceName {
	LABEL 	"|en|device name" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (16)  {
		DEFAULT_VALUE	"FMR 23x" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parOrderCode {
	LABEL 	"|en|order code" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (16)  {
		DEFAULT_VALUE	"" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMamFilterLength {
	LABEL 	"|en|MAM filt. length" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"2.1u" ;
		EDIT_FORMAT	"2.1u" ;
		MIN_VALUE 	1 ;
		MAX_VALUE	32 ;
		DEFAULT_VALUE	20 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMamFilterBorder {
	LABEL 	"|en|MAM filt. border" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"2.1u" ;
		EDIT_FORMAT	"2.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	15 ;
		DEFAULT_VALUE	3 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLowPassFilter {
	LABEL 	"|en|low pass filter" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|s" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.1f" ;
		EDIT_FORMAT	"5.1f" ;
		MIN_VALUE 	0.0 ;
		MAX_VALUE	255.0 ;
		DEFAULT_VALUE	5.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parHystereseWidth {
	LABEL 	"|en|hysterese width" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "8.3f";
					DISPLAY_FORMAT "8.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45: 1000.000;
						CASE 44:   39.370;
						CASE 47:   39.370;
						CASE 49: 1000.000;
						DEFAULT:   1000.000;
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEchoLostRampUp {
	LABEL 	"|en|echo lost up" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm/s";
					CASE 44:   "inch/s";
					CASE 47:   "inch/s";
					CASE 49:   "mm/s";
					DEFAULT:  "mm/s";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22000.0;
							CASE 44: 866.142;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22000.0;
						}
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEchoLostRampDown {
	LABEL 	"|en|echo lost down" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm/s";
					CASE 44:   "inch/s";
					CASE 47:   "inch/s";
					CASE 49:   "mm/s";
					DEFAULT:  "mm/s";
				}
	TYPE 	FLOAT	{
					EDIT_FORMAT "9.3f";
					DISPLAY_FORMAT "9.3f";
					MIN_VALUE 0.0;
					MAX_VALUE
					IF (parMaxAbs) {
                                                parAbsMaxMessDistance;
					}
					ELSE {
						SELECT (parDistanceUnit)
						{
							CASE 45: 22000.0;
							CASE 44: 866.142;
							CASE 47: 866.142;
							CASE 49: 22000.0;
							DEFAULT: 22000.0;
						}
					}
					DEFAULT_VALUE 0.0;
				}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parUnfilteredDistanceRaw {
	LABEL 	"|en|unflt. dist. raw" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parHKStatistik {
	LABEL 	"|en|env. statistics" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFilterBreite {
	LABEL 	"|en|env. smoothing" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"2.1u" ;
		EDIT_FORMAT	"2.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	77 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEnvelopeEnergy {
	LABEL 	"|en|envel. energy" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMappingAdder {
	LABEL 	"|en|mapping adder" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	5 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMappingWidth {
	LABEL 	"|en|mapping scanrate" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	1 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	6 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMappingAverage {
	LABEL 	"|en|mapping average" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	6 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFactoryTdt {
	LABEL 	"|en|fact. map. valid" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}, { 2 ,"|en|reset|de|loeschen|fr|reset|es|borrar|it|reset|nl|reset"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFacAdder {
	LABEL 	"|en|FAC adder" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFACWidth {
	LABEL 	"|en|FAC scan rate" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	1 ;
		MAX_VALUE	255 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parRangeFactoryMap {
	LABEL 	"|en|range fact. map" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFactoryMappingValid {
	LABEL 	"|en|fact. map. valid" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE 	ENUMERATED	{  { 0 ,"|en|not active"}, { 1 ,"|en|active"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCustomerMappingValid {
	LABEL 	"|en|cust. map. valid" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE 	ENUMERATED	{  { 0 ,"|en|not active"}, { 1 ,"|en|active"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEdgeDetectionMode {
	LABEL 	"|en|edge detec. mode" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|front of echo"}, { 1 ,"|en|behind echo"}, { 2 ,"|en|middle of echo"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEdgeParameter {
	LABEL 	"|en|edge parameter" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	4 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFirstEchoFactor {
	LABEL 	"|en|First Echo Fact." ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	20 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFefThreshold {
	LABEL 	"|en|FEF threshold" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFefNearFactor {
	LABEL 	"|en|FEF at near dist" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFefDistanceNear {
	LABEL 	"|en|FEF dist. near" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE	  0.0;
				MAX_VALUE
				IF (parMaxAbs) {
					parAbsMaxMessDistance;
				}
				ELSE
				{
					SELECT (parDistanceUnit)
					{
						CASE 45: 22000.0;
						CASE 44: 866.142;
						CASE 47: 866.142;
						CASE 49: 22000.0;
						DEFAULT: 22000.0;
					}
				}
				DEFAULT_VALUE 0.0;
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFefDistanceFar {
	LABEL 	"|en|FEF dist. far" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE	  0.0;
				MAX_VALUE
				IF (parMaxAbs) {
					parAbsMaxMessDistance;
				}
				ELSE
				{
					SELECT (parDistanceUnit)
					{
						CASE 45: 22000.0;
						CASE 44: 866.142;
						CASE 47: 866.142;
						CASE 49: 22000.0;
						DEFAULT: 22000.0;
					}
				}
				DEFAULT_VALUE 0.0;
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parFefEdge {
	LABEL 	"|en|FEF edge" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	20 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDishBottomDetection {
	LABEL 	"|en|Tank Bottom Det." ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 1; { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAmplLevelBottomDetection {
	LABEL 	"|en|min. ampl. TBD" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	70 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMinLevelBottomDetection {
	LABEL 	"|en|max. level TBD" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE	  0.0;
				MAX_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45:   22000.000;
						CASE 44:     866.142;
						CASE 47:     866.142;
						CASE 49:   22000.000;
						DEFAULT:     22000.000;
					};
				DEFAULT_VALUE
					SELECT (parDistanceUnit)
					{
						CASE 45:     500.000;
						CASE 44:      19.685;
						CASE 47:      19.685;
						CASE 49:     500.000;
						DEFAULT:       500.000;
					};

			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLevelSpeed {
	LABEL 	"|en|fill/drain speed" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm/s";
					CASE 44:   "inch/s";
					CASE 47:   "inch/s";
					CASE 49:   "mm/s";
					DEFAULT:  "mm/s";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"9.3f" ;
		EDIT_FORMAT	"9.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parHfModul {
	LABEL 	"|en|HF modul" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 1; { 0 ,"|en|P II.1"}, { 1 ,"|en|P II.2"}, { 2 ,"|en|P III.1"}, { 3 ,"|en|P III.2"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parZeroDistance {
	LABEL 	"|en|zero distance" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "8.3f";
				DISPLAY_FORMAT "8.3f";
				MIN_VALUE	  0.0;
				MAX_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   5000.000;
					CASE 44:    196.850;
					CASE 47:    196.850;
					CASE 49:   5000.000;
					DEFAULT:     5000.000;
				};
				DEFAULT_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   700.000;
					CASE 44:    27.559;
					CASE 47:    27.559;
					CASE 49:   700.000;
					DEFAULT:     700.000;
				};

			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAntennaType {
	LABEL 	"|en|antenna type" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 1; { 0 ,"|en|rod aerial"}, { 1 ,"|en|horn aerial"}, { 2 ,"|en|dish antenna"}, { 3 ,"|en|planar antenna"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parInactiveLength {
	LABEL 	"|en|inactive length" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   -5000.000;
					CASE 44:    -196.850;
					CASE 47:    -196.850;
					CASE 49:   -5000.000;
					DEFAULT:     -5000.000;
				};
				MAX_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   5000.000;
					CASE 44:    196.850;
					CASE 47:    196.850;
					CASE 49:   5000.000;
					DEFAULT:     5000.000;
				};
				DEFAULT_VALUE 0.000;
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parTotalLength {
	LABEL 	"|en|total length" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   -5000.000;
					CASE 44:    -196.850;
					CASE 47:    -196.850;
					CASE 49:   -5000.000;
					DEFAULT:     -5000.000;
				};
				MAX_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   5000.000;
					CASE 44:    196.850;
					CASE 47:    196.850;
					CASE 49:   5000.000;
					DEFAULT:     5000.000;
				};
				DEFAULT_VALUE 0.000;
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parPositionOfReferencePulse {
	LABEL 	"|en|pos. ref. pulse" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"8.3f" ;
		EDIT_FORMAT	"8.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAmplOfReferencePulse {
	LABEL 	"|en|ampl. ref. pulse" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  INTEGER (1)  {
		DISPLAY_FORMAT	"4.1i" ;
		EDIT_FORMAT	"4.1i" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parPlausibilityFunction {
	LABEL 	"|en|Plausib.Function" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|off|de|aus|fr|inactif|es|desconectado|it|disattivato|nl|uit"}, { 1 ,"|en|on|de|an|fr|actif|es|conectado|it|attivato|nl|aan"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMaxDeltaPlausibilityFunction {
	LABEL 	"|en|PF maximum delta" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE   0.0;
				MAX_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   5000.000;
					CASE 44:    196.850;
					CASE 47:    196.850;
					CASE 49:   5000.000;
					DEFAULT:     5000.000;
				};
				DEFAULT_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   500.000;
					CASE 44:    19.685;
					CASE 47:    19.685;
					CASE 49:   500.000;
					DEFAULT:     500.000;
				};
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parUpperAreaPlausibilityFunction {
	LABEL 	"|en|PF upper area" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "mm";
					CASE 44:   "inch";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
				EDIT_FORMAT "9.3f";
				DISPLAY_FORMAT "9.3f";
				MIN_VALUE   0.0;
				MAX_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   5000.000;
					CASE 44:    196.850;
					CASE 47:    196.850;
					CASE 49:   5000.000;
					DEFAULT:     5000.000;
				};
				DEFAULT_VALUE
				SELECT (parDistanceUnit)
				{
					CASE 45:   350.000;
					CASE 44:    13.780;
					CASE 47:    13.780;
					CASE 49:   350.000;
					DEFAULT:     350.000;
				};
			}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDisplayVersion {
	LABEL 	"|en|display version" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  PACKED_ASCII (32)  {
		DEFAULT_VALUE	"" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCalculatedCycleTime {
	LABEL 	"|en|calc. cycle time" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|ms" ;
	TYPE  UNSIGNED_INTEGER (2) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSwVersion {
	LABEL 	"|en|sw. vers. appl." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER (2) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parParamVersion {
	LABEL 	"|en|para. vers. appl" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER (2) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDAAdjust4mA {
	LABEL 	"|en|D/A adjust 4mA" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"5.1u" ;
		EDIT_FORMAT	"5.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	65535 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDAAdjust20mA {
	LABEL 	"|en|D/A adjust 20mA" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"5.1u" ;
		EDIT_FORMAT	"5.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	65535 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDaaIndex1 {
	LABEL 	"|en|debug index 1" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd008)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDaaIndex2 {
	LABEL 	"|en|debug index 2" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd008)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (1)  {
		DISPLAY_FORMAT	"3.1u" ;
		EDIT_FORMAT	"3.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	255 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDaaRdWrValue {
	LABEL 	"|en|debug value" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd008)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"10.1f" ;
		EDIT_FORMAT	"10.1f" ;
		MIN_VALUE 	-9999999.0 ;
		MAX_VALUE	9999999.0 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parErrorStateOfAlgorithm {
	LABEL 	"|en|err. state alg." ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER (4) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parStateOfAlgorithm {
	LABEL 	"|en|state algorithm" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	VALIDITY  IF (Val006)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER (4) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parActualError {
	LABEL 	"|en|present error|de|aktueller Fehler|fr|defaut actuel|es|aktueller Fehler|it|errore attuale|nl|actuele fout" ;
	HELP 	[error_code_help] ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED (2)	{
    {0, "|en|System OK"},
    {101, "|en|A101 checksum error general reset & new calibr.required|de|A101 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A101 erreur somme test reset gnral nouvel talonn.|it|A101 errore check sum ripristino totale nuova calibraz.  "},
    {102, "|en|A102 checksum error general reset & new calibr.required|de|A102 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A102 erreur somme test reset gnral nouvel talonn.|it|A102 errore check sum ripristino totale nuova calibraz.  "},
    {103, "|en|W103 initialising - please wait |de|W103 Initialisierung - bitte warten |fr|W103 initialisation en cours-attendre |it|W103 inizializzazione in corso-attendere   "},
    {106, "|en|A106 downloading please wait |de|A106 Download luft - bitte warten |fr|A106 download en cours-attendre |it|A106 download in corso-attendere   "},
    {110, "|en|A110 checksum error general reset & new calibr.required|de|A110 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A110 erreur somme test reset gnral nouvel talonn.|it|A110 errore di check sum riprist. totale nuova calibr.  "},
    {111, "|en|A111 electronics defect  |de|A111 Elektronik defekt  |fr|A111 dfaut lectronique  |it|A111 Guasto elettron.    "},
    {113, "|en|A113 electronics defect  |de|A113 Elektronik defekt  |fr|A113 dfaut lectronique  |it|A113 Guasto elettron.    "},
    {114, "|en|A114 electronics defect  |de|A114 Elektronik defekt  |fr|A114 dfaut lectronique  |it|A114 Guasto elettron.    "},
    {115, "|en|A115 electronics defect  |de|A115 Elektronik defekt  |fr|A115 dfaut lectronique  |it|A115 Guasto elettron.    "},
    {116, "|en|A116 download error repeat download |de|A116 Downloadfehler Download wiederholen |fr|A116 erreur download rpter download |it|A116 Errore download ripetere download   "},
    {121, "|en|A121 electronics defect  |de|A121 Elektronik defekt  |fr|A121 dfaut lectronique  |it|A121 Guasto elettron.    "},
    {152, "|en|A152 checksum error general reset & new calibr.required|de|A152 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A152 erreur somme test reset gnral nouvel talonn.|it|A152 errore check sum ripristino totale nuova calibraz.  "},
    {153, "|en|W153 initialising - please wait |de|W153 Initialisierung - bitte warten |fr|W153 initialisation en cours-attendre |it|W153 inizializzazione in corso-attendere   "},
    {155, "|en|A155 electronics defect  |de|A155 Elektronik defekt  |fr|A155 dfaut lectronique  |it|A155 Guasto elettron.    "},
    {160, "|en|A160 checksum error general reset & new calibr.required|de|A160 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A160 erreur somme test reset gnral nouvel talonn.|it|A160 errore check sum ripristino totale nuova calibraz.  "},
    {164, "|en|A164 electronics defect  |de|A164 Elektronik defekt  |fr|A164 dfaut lectronique  |it|A164 Guasto elettron.    "},
    {171, "|en|A171 electronics defect  |de|A171 Elektronik defekt  |fr|A171 dfaut lectronique  |it|A171 Guasto elettron.    "},
    {231, "|en|A231 sensor 1 defect check connection |de|A231 Sensor 1 defekt Prfe Verbindung |fr|A231 dfaut sonde 1 vrifier connexion |it|A231 Sens.1 guasto controll. conness.   "},
    {270, "|en|A270 custody switch undef check position |de|A270 Schalter undefiniert berprfe Position |fr|A270 commutateur indfini vrifier position |it|A270 Interr. non def. contr. posizione   "},
    {271, "|en|W271 reduced accuracy check installation |de|W271 reduz. Genauigkeit Installation prfen |fr|W271 prcision rduit vrifier installat. |it|W271 precis. ridotto controll. Installaz.   "},
    {272, "|en|A272 electronics defect amplifier |de|A272 Elektronik defekt Verstrker |fr|A272 dfaut lectron. amplificateur |it|A272 guasto elettron. amplificatore   "},
    {273, "|en|W273 reduced accuracy  |de|W273 reduz. Genauigkeit  |fr|W273 prcision rduit  |it|W273 precis. ridotto    "},
    {274, "|en|W274 reduced accuracy check pipe diameter |de|W274 reduz. Genauigkeit Prfe Rohr- durchmesser|fr|W274 prcision rduit vr. diamtre du tube|it|W274 precis. ridotto controll. diametro del tubo guida  "},
    {275, "|en|W275 electronics defect factory settings |de|W275 Elektronik defekt Werksabgleich |fr|W275 dfaut lectron. calibration usine |it|W275 dfaut lectron. calibrazione di fabbrica  "},
    {511, "|en|W511 no factory cali- bration ch1 |de|W511 kein Werksabgl. vorhanden K1 |fr|W511 pas de rglage usine disponible voie 1 |it|W511 assenza calibr. di fabbrica C1   "},
    {512, "|en|W512 recording of mapping please wait |de|W512 Aufnahme Ausblendung - warten |fr|W512 suppressionen cours attendre |it|W512 registr. mappatura in corso-attendere   "},
    {601, "|en|W601 linearisation ch1 curve not monotone |de|W601 Linearisierung K1 Kurve nicht monoton |fr|W601 linarisation voie 1 courbe non monotone |it|W601 Linearizzazione C1 curva non monotona   "},
    {611, "|en|W611 less than 2 linea- risation points for channel 1|de|W611 Linearisierungspkt. Anzahl <2 (K1) |fr|W611 linarisation voie 1 moins de 2 points |it|W611 N punti linearizz. minori di 2 (C1)   "},
    {621, "|en|W621 simulation ch. 1 on |de|W621 Simulation K1 eingeschaltet |fr|W621 simulation voie 1 active |it|W621 Simulazione C1 attiva   "},
    {641, "|en|E641 no usable echo channel 1 check calibr.|de|E641 kein auswertbares Echo K1 Abgleich prfen|fr|E641 pas d'cho valuable en voie 1 vrifier calib.|it|E641 mancanza eco per elabor. C1 controll. cali.  "},
    {651, "|en|E651 level in safety distance - risk of overspill|de|E651 Sicherheitsabst. erreicht berfllgefahr|fr|E651 distance de scurit atteinte danger dbord.|it|E651 Raggiunta distanza di sicurezza Pericolo tracim  "},
    {671, "|en|A671 linearisation ch1 not complete, not usable|de|A671 Linearisation Ch1 nicht vollstndig, unbrauchbar|fr|A671 linaristion voie incomplte, inutilisable|it|A671 linearizzaz. C1 non completa, non utilizzabile  "},
    {681, "|en|W681 current ch1 out of range |de|W681 Strom Ch1 ausserhalb des Messbereichs |fr|W681 courant voie hors gamme |it|W681 usc. corr. C1 fuori range   "}}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parLastError {
	LABEL 	"|en|previous error|de|letzter Fehler|fr|dernier defaut|es|letzter Fehler|it|ultimo errore|nl|vorige fout" ;
	HELP 	[error_code_help] ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED (2)	{
    {0, "|en|System OK"},
    {101, "|en|A101 checksum error general reset & new calibr.required|de|A101 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A101 erreur somme test reset gnral nouvel talonn.|it|A101 errore check sum ripristino totale nuova calibraz.  "},
    {102, "|en|A102 checksum error general reset & new calibr.required|de|A102 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A102 erreur somme test reset gnral nouvel talonn.|it|A102 errore check sum ripristino totale nuova calibraz.  "},
    {103, "|en|W103 initialising - please wait |de|W103 Initialisierung - bitte warten |fr|W103 initialisation en cours-attendre |it|W103 inizializzazione in corso-attendere   "},
    {106, "|en|A106 downloading please wait |de|A106 Download luft - bitte warten |fr|A106 download en cours-attendre |it|A106 download in corso-attendere   "},
    {110, "|en|A110 checksum error general reset & new calibr.required|de|A110 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A110 erreur somme test reset gnral nouvel talonn.|it|A110 errore di check sum riprist. totale nuova calibr.  "},
    {111, "|en|A111 electronics defect  |de|A111 Elektronik defekt  |fr|A111 dfaut lectronique  |it|A111 Guasto elettron.    "},
    {113, "|en|A113 electronics defect  |de|A113 Elektronik defekt  |fr|A113 dfaut lectronique  |it|A113 Guasto elettron.    "},
    {114, "|en|A114 electronics defect  |de|A114 Elektronik defekt  |fr|A114 dfaut lectronique  |it|A114 Guasto elettron.    "},
    {115, "|en|A115 electronics defect  |de|A115 Elektronik defekt  |fr|A115 dfaut lectronique  |it|A115 Guasto elettron.    "},
    {116, "|en|A116 download error repeat download |de|A116 Downloadfehler Download wiederholen |fr|A116 erreur download rpter download |it|A116 Errore download ripetere download   "},
    {121, "|en|A121 electronics defect  |de|A121 Elektronik defekt  |fr|A121 dfaut lectronique  |it|A121 Guasto elettron.    "},
    {152, "|en|A152 checksum error general reset & new calibr.required|de|A152 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A152 erreur somme test reset gnral nouvel talonn.|it|A152 errore check sum ripristino totale nuova calibraz.  "},
    {153, "|en|W153 initialising - please wait |de|W153 Initialisierung - bitte warten |fr|W153 initialisation en cours-attendre |it|W153 inizializzazione in corso-attendere   "},
    {155, "|en|A155 electronics defect  |de|A155 Elektronik defekt  |fr|A155 dfaut lectronique  |it|A155 Guasto elettron.    "},
    {160, "|en|A160 checksum error general reset & new calibr.required|de|A160 Prfsummenfehler Totalreset & Neu- abgl. erfordert|fr|A160 erreur somme test reset gnral nouvel talonn.|it|A160 errore check sum ripristino totale nuova calibraz.  "},
    {164, "|en|A164 electronics defect  |de|A164 Elektronik defekt  |fr|A164 dfaut lectronique  |it|A164 Guasto elettron.    "},
    {171, "|en|A171 electronics defect  |de|A171 Elektronik defekt  |fr|A171 dfaut lectronique  |it|A171 Guasto elettron.    "},
    {231, "|en|A231 sensor 1 defect check connection |de|A231 Sensor 1 defekt Prfe Verbindung |fr|A231 dfaut sonde 1 vrifier connexion |it|A231 Sens.1 guasto controll. conness.   "},
    {270, "|en|A270 custody switch undef check position |de|A270 Schalter undefiniert berprfe Position |fr|A270 commutateur indfini vrifier position |it|A270 Interr. non def. contr. posizione   "},
    {271, "|en|W271 reduced accuracy check installation |de|W271 reduz. Genauigkeit Installation prfen |fr|W271 prcision rduit vrifier installat. |it|W271 precis. ridotto controll. Installaz.   "},
    {272, "|en|A272 electronics defect amplifier |de|A272 Elektronik defekt Verstrker |fr|A272 dfaut lectron. amplificateur |it|A272 guasto elettron. amplificatore   "},
    {273, "|en|W273 reduced accuracy  |de|W273 reduz. Genauigkeit  |fr|W273 prcision rduit  |it|W273 precis. ridotto    "},
    {274, "|en|W274 reduced accuracy check pipe diameter |de|W274 reduz. Genauigkeit Prfe Rohr- durchmesser|fr|W274 prcision rduit vr. diamtre du tube|it|W274 precis. ridotto controll. diametro del tubo guida  "},
    {275, "|en|W275 electronics defect factory settings |de|W275 Elektronik defekt Werksabgleich |fr|W275 dfaut lectron. calibration usine |it|W275 dfaut lectron. calibrazione di fabbrica  "},
    {511, "|en|W511 no factory cali- bration ch1 |de|W511 kein Werksabgl. vorhanden K1 |fr|W511 pas de rglage usine disponible voie 1 |it|W511 assenza calibr. di fabbrica C1   "},
    {512, "|en|W512 recording of mapping please wait |de|W512 Aufnahme Ausblendung - warten |fr|W512 suppressionen cours attendre |it|W512 registr. mappatura in corso-attendere   "},
    {601, "|en|W601 linearisation ch1 curve not monotone |de|W601 Linearisierung K1 Kurve nicht monoton |fr|W601 linarisation voie 1 courbe non monotone |it|W601 Linearizzazione C1 curva non monotona   "},
    {611, "|en|W611 less than 2 linea- risation points for channel 1|de|W611 Linearisierungspkt. Anzahl <2 (K1) |fr|W611 linarisation voie 1 moins de 2 points |it|W611 N punti linearizz. minori di 2 (C1)   "},
    {621, "|en|W621 simulation ch. 1 on |de|W621 Simulation K1 eingeschaltet |fr|W621 simulation voie 1 active |it|W621 Simulazione C1 attiva   "},
    {641, "|en|E641 no usable echo channel 1 check calibr.|de|E641 kein auswertbares Echo K1 Abgleich prfen|fr|E641 pas d'cho valuable en voie 1 vrifier calib.|it|E641 mancanza eco per elabor. C1 controll. cali.  "},
    {651, "|en|E651 level in safety distance - risk of overspill|de|E651 Sicherheitsabst. erreicht berfllgefahr|fr|E651 distance de scurit atteinte danger dbord.|it|E651 Raggiunta distanza di sicurezza Pericolo tracim  "},
    {671, "|en|A671 linearisation ch1 not complete, not usable|de|A671 Linearisation Ch1 nicht vollstndig, unbrauchbar|fr|A671 linaristion voie incomplte, inutilisable|it|A671 linearizzaz. C1 non completa, non utilizzabile  "},
    {681, "|en|W681 current ch1 out of range |de|W681 Strom Ch1 ausserhalb des Messbereichs |fr|W681 courant voie hors gamme |it|W681 usc. corr. C1 fuori range   "}}

} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parClearLastError {
	LABEL 	"|en|clear last error|de|Loesche let.Fehl.|fr|eff. der. defaut|es|Loesche let.Fehl.|it|canc.ult. err.|nl|wis laatste fout" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|keep|de|beibehalten|fr|garder|es|mantener|it|mantieni|nl|houden"}, { 1 ,"|en|erase|de|loeschen|fr|effacer|es|borrar|it|cancella|nl|wissen"}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parReset {
	LABEL 	"|en|reset|de|Ruecksetzen|fr|Remise  zero|es|Ruecksetzen|it|reset|nl|reset" ;
	HELP 	"|en|for reset codesee manual|de|Zur Codeeingabesiehe Betriebsanl.|fr|pour code r..z.voir mise en serv.|es|cdigo de borradover manual|it|per codice resetvedi manuale|nl|voor resetcodezie handboek" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"5.1u" ;
		EDIT_FORMAT	"5.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	65535 ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMeasuredDistance {
	LABEL 	"|en|measured dist.|de|gemessene Dist.|fr|distance mesuree|es|gemessene Dist.|it|Distanza mis.|nl|meetafstand" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"6.3f" ;
		EDIT_FORMAT	"6.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMeasuredLevel {
	LABEL 	"|en|measured level|de|gemess. Fuellst.|fr|niveau mesure|es|gemess. Fuellst.|it|Livello mis.|nl|gemeten niveau" ;
	CLASS 	COMPUTATION & DYNAMIC ; 
	HANDLING  READ ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"6.3f" ;
		EDIT_FORMAT	"6.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parApplicationParameter {
	LABEL 	"|en|application par.|de|Anwendungsparam.|fr|par. d'applicat.|es|Anwendungsparam.|it|par. applicaz.|nl|toepassing par." ;
	CLASS 	INPUT ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; 
	{ 0 ,"|en|not modified|de|nicht geaendert|fr|non modifie|es|non modificato|it|non modificato|nl|niet verander."},
	{ 1 ,"|en|modified|de|geaendert|fr|modifie|es|modificato|it|modificato|nl|veranderet"}
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parTagNumber {
	LABEL 	"|en|tag no.|de|Messstelle|fr|N Repere|es|Messstelle|it|Tag n|nl|tag nummer" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (16) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parProtSoftVersionString {
	LABEL 	"|en|protocol+sw-no.|de|Protokoll+SW-Nr.|fr|protoc. +nsw|es|Protokoll+SW-Nr.|it|protocol.+sw ver|nl|protoc.en SW.nr." ;
	CLASS 	INPUT ; 
	HANDLING  READ ;
	TYPE  PACKED_ASCII (16)  {
		DISPLAY_FORMAT	"V01.01.0x HART" ;
		DEFAULT_VALUE	"V01.01.0x HART" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSerialNumber {
	LABEL 	"|en|serial no.|de|Seriennummer|fr|numero de serie|es|Seriennummer|it|Num. di serie|nl|serienummer" ;
	CLASS 	INPUT ; 
	HANDLING  READ ;
	TYPE  PACKED_ASCII (16) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDownloadMode {
	LABEL 	"|en|download mode|de|Download Mode|fr|mode download|es|Download Mode|it|modo download|nl|download keuze" ;
	CLASS 	INPUT ; 
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 0; { 0 ,"|en|parameter only|de|nur Parameter|fr|seul. param.|es|slo parmet.|it|solo parametro|nl|alleen param."}, { 1 ,"|en|param+cust.map|de|Param+Ausblen.|fr|param.+ suppr.|es|par.+mapeado|it|param+map.cl.|nl|param.+select."}, { 2 ,"|en|mapping only|de|Ausblendung|fr|suppression|es|slo mapeado|it|solo mappatura|nl|alleen select."}  }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSimulationValueVolume {
	LABEL 	"sim. value volume" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val014)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ & WRITE ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
				{
					CASE 45:   "m";
					CASE 44:   "ft";
					CASE 47:   "inch";
					CASE 49:   "mm";
					DEFAULT:  "mm";
				}
	TYPE 	FLOAT	{
	EDIT_FORMAT "9.3f";
	DISPLAY_FORMAT "9.3f";
	MIN_VALUE
	IF ((parLevelUllageMode == 0) || (parLevelUllageMode == 2))
	{
		-99999.0;
	}
	ELSE
	{
		IF ((parLevelUllageMode == 1) || (parLevelUllageMode == 3))
		{
			SELECT (parDistanceUnit)
			{
				CASE 45 :  -2.000;
				CASE 44 :  -6.562;
				CASE 47 :  -78.740;
				CASE 49 :  -2000.000;
				DEFAULT:  -2.000;
			}
		}
	}
	MAX_VALUE
	IF ((parLevelUllageMode == 0) || (parLevelUllageMode == 2))
	{
		99999.0;
	}
	ELSE
	{
		IF ((parLevelUllageMode == 1) || (parLevelUllageMode == 3))
		{
			IF (parMaxAbs) {
				parAbsMaxMessDistance;
			}
			ELSE {
				SELECT (parDistanceUnit)
				{
					CASE 45: 22.0;
					CASE 44: 72.178;
					CASE 47: 866.142;
					CASE 49: 22000.0;
					DEFAULT: 22.0;
				}
			}
		}
	}
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  preDistanceUnit {
	LABEL 	"Internal" ;
	CLASS 	LOCAL ; 
	HANDLING  READ & WRITE ;
	TYPE  UNSIGNED_INTEGER (2)  {
		DEFAULT_VALUE	45 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  MEASVAR_LOWER_LIMIT {
	LABEL 	"Internal" ;
	CLASS 	LOCAL ; 
	TYPE  FLOAT  {
		DEFAULT_VALUE	0 ;
		INITIAL_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  MEASVAR_UPPER_LIMIT {
	LABEL 	"Internal" ;
	CLASS 	LOCAL ; 
	TYPE  FLOAT  {
		DEFAULT_VALUE	100 ;
		INITIAL_VALUE	100 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  manufacturer_id {
	LABEL 	[manufacturer_id] ;
	HELP 	[manufacturer_id_code_help] ;
	CLASS 	HART ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	{ DEFAULT_VALUE 17;
	{17, "Endress + Hauser"}
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  universal_revision {
	LABEL 	[universal_revision] ;
	HELP 	[universal_revision_help] ;
	CLASS 	HART ; 
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER  {
		MAX_VALUE	253 ;
		DEFAULT_VALUE	5 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  software_revision {
	LABEL 	[software_revision] ;
	HELP 	[field_device_software_revision_help] ;
	CLASS 	DEVICE ; 
	HANDLING  READ ;
	TYPE  UNSIGNED_INTEGER  {
		MAX_VALUE	253 ;
	}
	POST_READ_ACTIONS  { 
		PostReadSoftware_revision
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  tag {
	LABEL 	[tag] ;
	HELP 	[tag_help] ;
	CLASS 	DEVICE ; 
	HANDLING  IF (Hnd011)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (8) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  message {
	LABEL 	[message] ;
	HELP 	[message_help] ;
	CLASS 	DEVICE ; 
	HANDLING  IF (Hnd011)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (32) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  descriptor {
	LABEL 	[descriptor] ;
	HELP 	[discriptor_help] ;
	CLASS 	DEVICE ; 
	HANDLING  IF (Hnd011)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  PACKED_ASCII (16) ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  date {
	LABEL 	[date] ;
	HELP 	[date_help] ;
	CLASS 	DEVICE ; 
	HANDLING  IF (Hnd011)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  DATE ;
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  final_assembly_number {
	LABEL 	[final_assembly_number] ;
	HELP 	[final_assembly_number_help] ;
	CLASS 	DEVICE ; 
	HANDLING  IF (Hnd011)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (3)  {
		EDIT_FORMAT	"8u"  ;
		MAX_VALUE	16777215 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  preOperationCode {
	LABEL 	"|en|unlock parameter|de|Freigabecode|fr|code operation|es|Freigabecode|it|Modo operativo|nl|startcode" ;
	CLASS 	LOCAL ; 
	HANDLING  IF (Hnd012)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (2)  {
		DISPLAY_FORMAT	"5.1u" ;
		EDIT_FORMAT	"5.1u" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	33997 ;
		DEFAULT_VALUE	100 ;
		INITIAL_VALUE	100 ;
	}
	POST_EDIT_ACTIONS  { 
		preOperationCodeModified
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parShowAccess {
	LABEL 	"|en|Access rights|de|Zugriff|fr|Droits d'accs" ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   1; 
	{ 0, "|en|Locked|de|Verriegelt|fr|Verrouill"}, 
	{ 1 ,"|en|Customer|de|Kunde|fr|Client"},
	{ 2 ,"|en|Service (View only)|de|Service (nur Ansicht)|fr|Service (Consultation)"}, 
	{ 3 ,"|en|Service|de|Service|fr|Service"},
	{ 5, "|en|WHG|de|WHG|fr|WHG"},
	{ 6, "|en|Debug unlock|de|Debug unlock|fr|Debug unlock"},
	{ 7, "|en|Hard lock|de|Hard lock|fr|Hard lock"}
}

} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMaxAbs {
	LABEL 	" " ;
	CLASS 	LOCAL ; 
	HANDLING  READ & WRITE ;
	TYPE  UNSIGNED_INTEGER (1)  {
		DEFAULT_VALUE	0 ;
		INITIAL_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parBarLowerRange {
	LABEL 	"Lower range" ;
	CLASS 	LOCAL ; 
	TYPE  FLOAT  {
		DEFAULT_VALUE	0.0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parSoftwareVersion {
	LABEL 	"Software revision" ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   0; 
        { 0, "Unknown"},
        { 1, "V1"},
        { 2, "V2"}
}

} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parAntennaExtensionLength {
	LABEL 	"|en|antenna extens.|de|Antenn. verlng" ;
	HELP 	"|en|length FAR10 - for FMR230 only|de|Lnge FAR10 - nur fr FMR230" ;
	CLASS 	CONTAINED ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	CONSTANT_UNIT  SELECT (parDistanceUnit)
                                {
                                        CASE 45:   "m";
                                        CASE 44:   "ft";
                                        CASE 47:   "inch";
                                        CASE 49:   "mm";
                                        DEFAULT:  "m";
                                }
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.3f" ;
		EDIT_FORMAT	"5.3f" ;
		MIN_VALUE 	0 ;
		MAX_VALUE	SELECT (parDistanceUnit)
{
        CASE 45: 5.0;
        CASE 44: 16.404;
        CASE 47: 866.142;
        CASE 49: 5000.0;
        DEFAULT: 5.0;
}

		DEFAULT_VALUE	0 ;
		INITIAL_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parMaxEchoSN {
	LABEL 	"|en|max.ampl.o.FAC" ;
	CLASS 	CONTAINED ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  "|en|dB" ;
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"3.0f" ;
		EDIT_FORMAT	"3.0f" ;
		DEFAULT_VALUE	0 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parEditRangeMaxSampleDist {
	LABEL 	"|en|max sample dist." ;
	CLASS 	CONTAINED ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	CONSTANT_UNIT  SELECT (parDistanceUnit)
                                {
                                        CASE 45:   "m";
                                        CASE 44:   "ft";
                                        CASE 47:   "inch";
                                        CASE 49:   "mm";
                                        DEFAULT:  "m";
                                }
	TYPE  FLOAT  {
		DISPLAY_FORMAT	"5.3f" ;
		EDIT_FORMAT	"5.3f" ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parWireType {
	LABEL 	"|en|wire type" ;
	CLASS 	CONTAINED ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  READ ;
	TYPE 	ENUMERATED	{  DEFAULT_VALUE   0; 
        { 0, "2 wire"},
        { 1, "4 wire"}
}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parDeviceIDNumber {
	LABEL 	"|en|device ID number" ;
	CLASS 	CONTAINED ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd003)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE  UNSIGNED_INTEGER (4)  {
		MIN_VALUE 	0 ;
		MAX_VALUE	16777215 ;
	}
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  parCurrentOutputMode {
	LABEL 	"|en|curr.output mode|de|Stromausg. Modus|fr|Sortie courant|es|Stromausg. Modus|it|modo uscita corr|nl|StroomUitg. mode" ;
	CLASS 	INPUT ; 
	VALIDITY  IF (Val016)
				{ TRUE; }
			ELSE
				{ FALSE; }
	HANDLING  IF (Hnd000)
				{ READ&WRITE; }
			ELSE
				{ READ; }
	TYPE 	ENUMERATED	{  
   { 0 ,"|en|standard|de|Standard|fr|Standard|es|estndar|it|standard|nl|standaard"}, 
   { 1 ,"|en|curr.turn down|de|Stromlupe|fr|Zoom courant|es|lazo de mA|it|corrente min.|nl|Stroomkring"}, 
   { 2 ,"|en|fixed current|de|fester Strom|fr|Courant fixe|es|salida mA fija|it|Corrente fissa|nl|vaste stroom"}  
}
} 



/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleBasicSetup {
	LABEL 	[LabTitleBasicSetup] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleBasicSetup] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleSafetySettings {
	LABEL 	[LabTitleSafetySettings] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleSafetySettings] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleLinearisation {
	LABEL 	[LabTitleLinearisation] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleLinearisation] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleExtendedCalibration {
	LABEL 	[LabTitleExtendedCalibration] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleExtendedCalibration] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleProfibusParameter {
	LABEL 	[LabTitleProfibusParameters] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleProfibusParameters] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleDisplay {
	LABEL 	[LabTitleDisplay] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleDisplay] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleDiagnostics {
	LABEL 	[LabTitleDiagnostics] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleDiagnostics] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleSystemParameter {
	LABEL 	[LabTitleSystemParameter] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleSystemParameter] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  DummySetUnitToBus {
	LABEL 	"|en|set unit to bus|de|Setze Einh. Out" ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumSetUnitToBus] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleDiagnosticsReset {
	LABEL 	[LabTitleDiagnosticsReset] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleDiagnosticsReset] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleHartParameter {
	LABEL 	[LabTitleHartParameters] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleHartParameters] }
        }
} 


/*****************************************************************************/ 
/*****************************************************************************/ 

VARIABLE  TitleOperationCode {
	LABEL 	[LabTitleOperationCode] ;
	CLASS 	LOCAL ; 
	HANDLING  READ ;
	TYPE 	ENUMERATED	        {
                DEFAULT_VALUE 0;
                {       0,      [EnumTitleOperationCode] }
        }
} 



/* !Main Menu! *************************************************************************/
MENU   page_phys_write_locking_pb {
	LABEL 	[phys_write_locking] ;
	ITEMS { 
		method_set
	} 
}

MENU   Menu_DeviceSimulation {
	LABEL 	"Device Simulation" ;
	ITEMS { 
	} 
}

MENU   Menu_Adjustment {
	LABEL 	[EH_mnuDevice_Adjustment] ;
	ITEMS { 
	} 
}

MENU   Menu_Main_Specialist {
	LABEL 	[main_menu] ;
	ITEMS { 
		Menu_File,
		Menu_S_Device,
		Menu_S_View,
		Menu_S_Options,
		Menu_Help
	} 
}

MENU   Menu_S_Device {
	LABEL 	[menu_Device] ;
	ITEMS { 
		OnlineDialog_SetAdress,
		SEPARATOR,
		std_LoadinDevice,
		std_LoadinPC,
		SEPARATOR,
		OnlineDialog_hart_comm,
		std_CommWay,
		configuration_flag_reset,
		SEPARATOR,
		OnlineDialog_SimCurroutp,
		OnlineDialog_SystemReset,
		SEPARATOR,
		OnlineDialog_Adjust_BasicSetup,
		OnlineDialog_Adjust_SafetySettings,
		OnlineDialog_Adjust_Linearisation,
		OnlineDialog_Adjust_ExtendedCalibration,
		OnlineDialog_Adjust_HartParam,
		OnlineDialog_Adjust_Display,
		OnlineDialog_Adjust_Diagnostics,
		OnlineDialog_Adjust_SystemParam
		IF (Val006) {
			OnlineDialog_SERVICE_MENU
		}
		SEPARATOR,
		OnlineDialog_write_locking
	} 
}

MENU   OnlineDialog_SetAdress {
	LABEL 	"|en|Set Adress|de|Adresse setzen" ;
	ITEMS { 
		parCommAddress
	} 
}

MENU   OnlineDialog_SimCurroutp {
	LABEL 	[Online_Simulation] ;
	ITEMS { 
		parSimulation,
		parSimulationValueLevel,
		parSimulationValueVolume,
		parSimulationValueCurrent
	} 
}

MENU   OnlineDialog_SystemReset {
	LABEL 	[Online_Reset] ;
	ITEMS { 
		parReset
	} 
}

MENU   OnlineDialog_Adjust_BasicSetup {
	LABEL 	[Online_Adjust_BasicSetup] ;
	ITEMS { 
		page_Basic_Setup_1_4,
		page_Basic_Setup_2_4,
		page_Basic_Setup_3_4,
		page_Basic_Setup_4_4
	} 
}

MENU   page_Basic_Setup_1_4 {
	LABEL 	[Online_Adjust_BasicSetup_1_4] ;
	ITEMS { 
		parProtSoftVersionString,
		parTagNumber,
		parDeviceName,
		parOrderCode
	} 
}

MENU   page_Basic_Setup_2_4 {
	LABEL 	[Online_Adjust_BasicSetup_2_4] ;
	ITEMS { 
		parTankShape,
		parDielectricConstant,
		parProcessCondition,
		parTubeDiameter
	} 
}

MENU   page_Basic_Setup_3_4 {
	LABEL 	[Online_Adjust_BasicSetup_3_4] ;
	ITEMS { 
		parEmptyCalibration,
		parFullCalibration
	} 
}

MENU   page_Basic_Setup_4_4 {
	LABEL 	[Online_Adjust_BasicSetup_4_4] ;
	ITEMS { 
		parMeasuredDistance,
		parCheckDistance,
		parSuppressionDistance,
		parStartMappingRecord,
		parActualError
	} 
}

MENU   OnlineDialog_Adjust_SafetySettings {
	LABEL 	[Online_Adjust_SafetySettings] ;
	ITEMS { 
		page_Safety_Settings_1_4,
		page_Safety_Settings_2_4,
		page_Safety_Settings_3_4,
		page_Safety_Settings_4_4
	} 
}

MENU   page_Safety_Settings_1_4 {
	LABEL 	[Online_Adjust_SafetySettings_1_4] ;
	ITEMS { 
		parOutputOnAlarm,
		parOutputAtError
	} 
}

MENU   page_Safety_Settings_2_4 {
	LABEL 	[Online_Adjust_SafetySettings_2_4] ;
	ITEMS { 
		parReactionLostEcho,
		parRampInPercentPerMin,
		parDelayTimeOnLostEcho
	} 
}

MENU   page_Safety_Settings_3_4 {
	LABEL 	[Online_Adjust_SafetySettings_3_4] ;
	ITEMS { 
		parInSafetyDistance,
		parLevelWithinSafetyDistance,
		parAcknowledgeAlarm
	} 
}

MENU   page_Safety_Settings_4_4 {
	LABEL 	[Online_Adjust_SafetySettings_4_4] ;
	ITEMS { 
		parOperationMode
	} 
}

MENU   OnlineDialog_Adjust_Linearisation {
	LABEL 	[Online_Adjust_Linearisation] ;
	ITEMS { 
		page_Linearisation_1_3,
		page_Linearisation_2_3,
		page_Linearisation_3_3
	} 
}

MENU   page_Linearisation_1_3 {
	LABEL 	[Online_Adjust_Linearisation_1_3] ;
	ITEMS { 
		parLevelUllageMode,
		parLinearisation
	} 
}

MENU   page_Linearisation_2_3 {
	LABEL 	[Online_Adjust_Linearisation_2_3] ;
	ITEMS { 
		parCylinderVessel,
		parCustomerUnit,
		parMaxVolume,
		parTableNumber,
		parInputLevelManual,
		parInputLevelHalfAutomatic,
		parInputVolume
	} 
}

MENU   page_Linearisation_3_3 {
	LABEL 	[Online_Adjust_Linearisation_3_3] ;
	ITEMS { 
		parLinearisation,
		parActualError
	} 
}

MENU   OnlineDialog_Adjust_ExtendedCalibration {
	LABEL 	[Online_Adjust_ExtendedCalibration] ;
	ITEMS { 
		page_Extended_Calibration_1_3,
		page_Extended_Calibration_2_3,
		page_Extended_Calibration_3_3
	} 
}

MENU   page_Extended_Calibration_1_3 {
	LABEL 	[Online_Adjust_ExtendedCalibration_1_3] ;
	ITEMS { 
		parMeasuredDistance,
		parCheckDistance,
		parSuppressionDistance,
		parStartMappingRecord,
		parActualError
	} 
}

MENU   page_Extended_Calibration_2_3 {
	LABEL 	[Online_Adjust_ExtendedCalibration_2_3] ;
	ITEMS { 
		parPresMapRange,
		parCustTankMap
	} 
}

MENU   page_Extended_Calibration_3_3 {
	LABEL 	[Online_Adjust_ExtendedCalibration_3_3] ;
	ITEMS { 
		software_revision(HIDDEN),
		parEchoQuality,
		parOffsetOfMeasuredDistance,
		parOutputDamping,
		parBlockingDistance,
		parAntennaExtensionLength
	} 
}

MENU   OnlineDialog_Adjust_HartParam {
	LABEL 	[Online_Adjust_HartParam] ;
	ITEMS { 
		page_Hart_Param_1_3,
		page_Hart_Param_2_3,
		page_Hart_Param_3_3
	} 
}

MENU   page_Hart_Param_1_3 {
	LABEL 	[Online_Adjust_ProfibusParam_1_3] ;
	ITEMS { 
		parCommAddress,
		parPreambles
	} 
}

MENU   page_Hart_Param_2_3 {
	LABEL 	[Online_Adjust_ProfibusParam_2_3] ;
	ITEMS { 
		software_revision(HIDDEN),
		parThresholdMainValue,
		parFixedCurrentOutput,
		parCurrentOutputMode,
		parFixedCurrentValue
	} 
}

MENU   page_Hart_Param_3_3 {
	LABEL 	[Online_Adjust_ProfibusParam_3_3] ;
	ITEMS { 
		parSimulation,
		parSimulationValueLevel,
		parSimulationValueVolume,
		parSimulationValueCurrent
	} 
}

MENU   OnlineDialog_Adjust_Display {
	LABEL 	[Online_Adjust_Display] ;
	ITEMS { 
		page_Display_1_1
	} 
}

MENU   page_Display_1_1 {
	LABEL 	[Online_Adjust_Display_1_1] ;
	ITEMS { 
		parLanguage,
		parBackToHome,
		parFormatDisplay_ft,
		parNoOfDecimals,
		parSeparationCharacter
	} 
}

MENU   OnlineDialog_Adjust_Diagnostics {
	LABEL 	[Online_Adjust_Diagnostics] ;
	ITEMS { 
		page_Diagnostics_1_2,
		page_Diagnostics_2_2
	} 
}

MENU   page_Diagnostics_1_2 {
	LABEL 	[Online_Adjust_Diagnostics_1_2] ;
	ITEMS { 
		parActualError,
		parLastError,
		parClearLastError,
		TitleDiagnosticsReset
	} 
}

MENU   page_Diagnostics_2_2 {
	LABEL 	[Online_Adjust_Diagnostics_2_2] ;
	ITEMS { 
		parMeasuredLevel,
		parApplicationParameter
	} 
}

MENU   OnlineDialog_Adjust_SystemParam {
	LABEL 	[Online_Adjust_SystemParam] ;
	ITEMS { 
		page_System_Param_1_1
	} 
}

MENU   page_System_Param_1_1 {
	LABEL 	[Online_Adjust_SystemParam_1_1] ;
	ITEMS { 
		parTagNumber,
		parProtSoftVersionString,
		parSerialNumber,
		parDistanceUnit
	} 
}

MENU   OnlineDialog_SERVICE_MENU {
	LABEL 	"|en|service|de|Service|fr|Service|es|servicio|it|Service|nl|service" ;
	ITEMS { 
		Page_service_lv_0,
		Page_service_lv_1,
		Page_service_lv_2,
		Page_service_lv_3,
		Page_service_lv_4,
		Page_service_lv_5,
		Page_service_lv_6,
		Page_service_lv_7,
		Page_service_lv_8,
		Page_service_lv_12,
		Page_service_lv_13,
		Page_service_lv_15
	} 
}

MENU   Page_service_lv_0 {
	LABEL 	"|en|info|de|Info|fr|Info|es|Info|it|Info|nl|informatie" ;
	ITEMS { 
		parCurrentAmplitude,
		parAmplitudeOverSuppression,
		parAmplitudeOverFAC,
		parUnfilteredDistance,
		parCalcEdgeParameter,
		parCurrentFirstEchoFactor,
		parDeviceName,
		parOrderCode
	} 
}

MENU   Page_service_lv_1 {
	LABEL 	"|en|distance" ;
	ITEMS { 
		parMamFilterLength,
		parMamFilterBorder,
		parLowPassFilter,
		parHystereseWidth,
		parEchoLostRampUp,
		parEchoLostRampDown,
		parUnfilteredDistanceRaw
	} 
}

MENU   Page_service_lv_2 {
	LABEL 	"|en|envelope" ;
	ITEMS { 
		parHKStatistik,
		parFilterBreite,
		parEnvelopeEnergy
	} 
}

MENU   Page_service_lv_3 {
	LABEL 	"|en|mapping" ;
	ITEMS { 
		parMappingAdder,
		parMappingWidth,
		parMappingAverage,
		parFactoryTdt,
		parFacAdder,
		parFACWidth,
		parRangeFactoryMap,
		parFactoryMappingValid,
		parCustomerMappingValid
	} 
}

MENU   Page_service_lv_4 {
	LABEL 	"|en|edge" ;
	ITEMS { 
		parEdgeDetectionMode,
		parEdgeParameter,
		parCalcEdgeParameter
	} 
}

MENU   Page_service_lv_5 {
	LABEL 	"|en|first echo" ;
	ITEMS { 
		software_revision(HIDDEN),
		parFirstEchoFactor,
		parFefThreshold,
		parFefNearFactor,
		parFefDistanceNear,
		parFefDistanceFar,
		parFefEdge,
		parMaxEchoSN,
		parCurrentFirstEchoFactor
	} 
}

MENU   Page_service_lv_6 {
	LABEL 	"|en|tank bottom" ;
	ITEMS { 
		parDishBottomDetection,
		parAmplLevelBottomDetection,
		parMinLevelBottomDetection,
		parLevelSpeed
	} 
}

MENU   Page_service_lv_7 {
	LABEL 	"|en|module" ;
	ITEMS { 
		parHfModul,
		parZeroDistance
	} 
}

MENU   Page_service_lv_8 {
	LABEL 	"|en|antenna table" ;
	ITEMS { 
		software_revision(HIDDEN),
		parAntennaType,
		parInactiveLength,
		parTotalLength,
		parAbsMaxMessDistance,
		parPositionOfReferencePulse,
		parAmplOfReferencePulse,
		parEditRangeMaxSampleDist
	} 
}

MENU   Page_service_lv_12 {
	LABEL 	"|en|system 1" ;
	ITEMS { 
		software_revision(HIDDEN),
		parPlausibilityFunction,
		parMaxDeltaPlausibilityFunction,
		parUpperAreaPlausibilityFunction,
		parWireType,
		parDisplayVersion,
		parCalculatedCycleTime,
		parSwVersion,
		parParamVersion
	} 
}

MENU   Page_service_lv_13 {
	LABEL 	"|en|system 2" ;
	ITEMS { 
		software_revision(HIDDEN),
		parDAAdjust4mA,
		parDAAdjust20mA,
		parDeviceIDNumber
	} 
}

MENU   Page_service_lv_15 {
	LABEL 	"|en|debug" ;
	ITEMS { 
		parDaaIndex1,
		parDaaIndex2,
		parDaaRdWrValue,
		parErrorStateOfAlgorithm,
		parStateOfAlgorithm
	} 
}

MENU   OnlineDialog_write_locking {
	LABEL 	[phys_write_locking] ;
	ITEMS { 
		page_phys_write_locking_user
	} 
}

MENU   page_phys_write_locking_user {
	LABEL 	[phys_write_locking_user] ;
	ITEMS { 
		parShowAccess,
		parOperationCode,
		write_protect,
		method_set
	} 
}

MENU   Menu_S_View {
	LABEL 	[menu_View] ;
	ITEMS { 
		OnlineWindow_display,
		OnlineWindow_yt,
		OnlineWindow_diagnosis,
		SEPARATOR,
		std_Funktionsleiste,
		std_Statuszeile,
		SEPARATOR,
		std_Update
	} 
}

MENU   OnlineWindow_display {
	LABEL 	[menu_dialog_display_measure] ;
	ITEMS { 
		Page_display_measure
	} 
}

MENU   Page_display_measure {
	LABEL 	[menu_dialog_display_measure] ;
	ITEMS { 
		IF (Val015) {
			BarVal_measuredValue
		}
		group_1
	} 
}

MENU   BarVal_measuredValue {
	LABEL 	"|en|measured value|de|Messwert|fr|Valeur de mesure|es|Messwert|it|Valore di misura|nl|gemeten waarde" ;
	ITEMS { 
		parMeasuredValue,
		parBarLowerRange,
		parMaxVolume
	} 
}

MENU   group_1 {
	LABEL 	"" ;
	ITEMS { 
		Val_parMeasuredValue,
		Val_parMeasuredDistance,
		Val_parMeasuredLevel,
		SEPARATOR,
		Val_parCurrentOutput,
		Val_parEchoQuality,
		Val_parActualError
	} 
}

MENU   Val_parMeasuredValue {
	LABEL 	"|en|measured value|de|Messwert|fr|Valeur de mesure|es|Messwert|it|Valore di misura|nl|gemeten waarde" ;
	ITEMS { 
		parMeasuredValue(READ_ONLY)
	} 
}

MENU   Val_parMeasuredDistance {
	LABEL 	"|en|measured dist.|de|gemessene Dist.|fr|distance mesuree|es|gemessene Dist.|it|Distanza mis.|nl|meetafstand" ;
	ITEMS { 
		parMeasuredDistance(READ_ONLY)
	} 
}

MENU   Val_parMeasuredLevel {
	LABEL 	"|en|measured level|de|gemess. Fuellst.|fr|niveau mesure|es|gemess. Fuellst.|it|Livello mis.|nl|gemeten niveau" ;
	ITEMS { 
		parMeasuredLevel(READ_ONLY)
	} 
}

MENU   Val_parCurrentOutput {
	LABEL 	"|en|output current|de|Ausgangsstrom|fr|Sortie courant|es|Ausgangsstrom|it|uscita corrente|nl|uitgangsstroom" ;
	ITEMS { 
		parCurrentOutput(READ_ONLY)
	} 
}

MENU   Val_parEchoQuality {
	LABEL 	"|en|echo quality|de|Echoqualitaet|fr|Qualite echo|es|Echoqualitaet|it|Qualit eco|nl|echokwaliteit" ;
	ITEMS { 
		parEchoQuality(READ_ONLY)
	} 
}

MENU   Val_parActualError {
	LABEL 	"|en|present error|de|aktueller Fehler|fr|defaut actuel|es|aktueller Fehler|it|errore attuale|nl|actuele fout" ;
	ITEMS { 
		parActualError(READ_ONLY)
	} 
}

MENU   OnlineWindow_diagnosis {
	LABEL 	[menu_dialog_diagnosis] ;
	ITEMS { 
		OnlineWindow_diagnosis_device
	} 
}

MENU   OnlineWindow_diagnosis_device {
	LABEL 	[menu_ident_device] ;
	ITEMS { 
		tag(READ_ONLY),
		manufacturer_id(READ_ONLY),
		device_id(READ_ONLY),
		device_type(READ_ONLY),
		software_revision(READ_ONLY),
		hardware_revision(READ_ONLY),
		transmitter_revision(READ_ONLY),
		universal_revision(READ_ONLY),
		device_status(READ_ONLY)
	} 
}


/* !Matrix Menu! ***********************************************************************/
MENU   Table_Main_Specialist {
	LABEL 	DEFAULT_VALUE_phys_device_id ;
	ITEMS { 
		MenuMainMicroPilot_M,
		mnuEhIdent
	} 
}

MENU   MenuMainMicroPilot_M {
	LABEL 	[EH_Standard_View] ;
	ITEMS { 
		CALIBRATION,
		SAFETY_SETTINGS,
		LINEARISATION,
		EXTENDED_CALIBRATION,
		OUTPUT_MNU,
		DISPLAY_MNU,
		OPERATING_PARAMETER,
		SYSTEM_PARAMETER
		IF (Val006) {
			SERVICE_MENU
		}
	} 
}

MENU   CALIBRATION {
	LABEL 	"|en|basic setup|de|Grundabgleich|fr|Etalonnage base|es|ajustes bsicos|it|Calibrazione base|nl|basisinstelling" ;
	ITEMS { 
		TitleBasicSetup(READ_ONLY),
		parMeasuredValue(READ_ONLY),
		parTankShape,
		parDielectricConstant,
		parProcessCondition,
		parEmptyCalibration,
		parFullCalibration,
		parTubeDiameter
	} 
}

MENU   SAFETY_SETTINGS {
	LABEL 	"|en|safety settings|de|Sicherheitseinst.|fr|Rglages scurit|es|ajustes de fb.|it|imp. sicurezza|nl|veiligheidsinst." ;
	ITEMS { 
		TitleSafetySettings(READ_ONLY),
		parOutputOnAlarm,
		parOutputAtError,
		parReactionLostEcho,
		parRampInPercentPerMin,
		parDelayTimeOnLostEcho,
		parInSafetyDistance,
		parLevelWithinSafetyDistance,
		parAcknowledgeAlarm,
		parOperationMode(READ_ONLY)
	} 
}

MENU   LINEARISATION {
	LABEL 	"|en|linearisation|de|Linearisierung|fr|Linarisation|es|linearizacin|it|Linearizzazione|nl|linearisering" ;
	ITEMS { 
		TitleLinearisation(READ_ONLY),
		parLevelUllageMode(READ_ONLY),
		parLinearisation(READ_ONLY),
		parCustomerUnit(READ_ONLY),
		parTableNumber(READ_ONLY),
		parInputLevelManual(READ_ONLY),
		parInputLevelHalfAutomatic(READ_ONLY),
		parInputVolume(READ_ONLY),
		parMaxVolume(READ_ONLY),
		parCylinderVessel(READ_ONLY)
	} 
}

MENU   EXTENDED_CALIBRATION {
	LABEL 	"|en|extended calibr.|de|erweit. Abgleich|fr|Etalonnage tendu|es|calib. extendida|it|Altre calibr.|nl|uitgebreide kalib" ;
	ITEMS { 
		software_revision(HIDDEN),
		TitleExtendedCalibration(READ_ONLY),
		parCheckDistance(READ_ONLY),
		parSuppressionDistance(READ_ONLY),
		parStartMappingRecord(READ_ONLY),
		parPresMapRange(READ_ONLY),
		parCustTankMap(READ_ONLY),
		parEchoQuality(READ_ONLY),
		parOffsetOfMeasuredDistance,
		parOutputDamping,
		parBlockingDistance,
		parAntennaExtensionLength
	} 
}

MENU   OUTPUT_MNU {
	LABEL 	"|en|output|de|Ausgang|fr|Sortie|es|salida|it|Uscita|nl|uitgang" ;
	ITEMS { 
		software_revision(HIDDEN),
		TitleHartParameter(READ_ONLY),
		parCommAddress(READ_ONLY),
		parPreambles(READ_ONLY),
		parThresholdMainValue(READ_ONLY),
		parFixedCurrentOutput(READ_ONLY),
		parCurrentOutputMode(READ_ONLY),
		parFixedCurrentValue(READ_ONLY),
		parSimulation(READ_ONLY),
		parSimulationValueLevel(READ_ONLY),
		parSimulationValueVolume(READ_ONLY),
		parSimulationValueCurrent(READ_ONLY),
		parCurrentOutput(READ_ONLY)
	} 
}

MENU   DISPLAY_MNU {
	LABEL 	"|en|display|de|Anzeige|fr|Affichage|es|visualizador|it|Indicatore|nl|display" ;
	ITEMS { 
		TitleDisplay(READ_ONLY),
		parLanguage,
		parBackToHome,
		parFormatDisplay_ft,
		parNoOfDecimals,
		parSeparationCharacter
	} 
}

MENU   OPERATING_PARAMETER {
	LABEL 	"|en|diagnostics|de|Diagnose|fr|Diagnostic|es|diagnsticos|it|Param. operativi|nl|diagnose" ;
	ITEMS { 
		TitleDiagnostics(READ_ONLY),
		parActualError(READ_ONLY),
		parLastError(READ_ONLY),
		parClearLastError,
		TitleDiagnosticsReset(READ_ONLY),
		parShowAccess(READ_ONLY),
		parOperationCode(READ_ONLY),
		parMeasuredDistance(READ_ONLY),
		parMeasuredLevel(READ_ONLY),
		parApplicationParameter(READ_ONLY)
	} 
}

MENU   SYSTEM_PARAMETER {
	LABEL 	"|en|system parameters|de|System Parameter|fr|Paramtre systme|es|parm sistema|it|Param. di sistema|nl|systeemparameter" ;
	ITEMS { 
		TitleSystemParameter(READ_ONLY),
		collectionTag.parTagMPM,
		parProtSoftVersionString(READ_ONLY),
		parSerialNumber,
		parDistanceUnit,
		parDownloadMode(READ_ONLY)
	} 
}

MENU   SERVICE_MENU {
	LABEL 	"|en|service|de|Service|fr|Service|es|servicio|it|Service|nl|service" ;
	ITEMS { 
		service_lv_0,
		service_lv_1,
		service_lv_2,
		service_lv_3,
		service_lv_4,
		service_lv_5,
		service_lv_6,
		service_lv_7,
		service_lv_8,
		service_lv_12,
		service_lv_13,
		service_lv_15
	} 
}

MENU   service_lv_0 {
	LABEL 	"|en|info|de|Info|fr|Info|es|Info|it|Info|nl|informatie" ;
	ITEMS { 
		parCurrentAmplitude(READ_ONLY),
		parAmplitudeOverSuppression(READ_ONLY),
		parAmplitudeOverFAC(READ_ONLY),
		parUnfilteredDistance(READ_ONLY),
		parCalcEdgeParameter(READ_ONLY),
		parCurrentFirstEchoFactor(READ_ONLY),
		parDeviceName(READ_ONLY),
		parOrderCode(READ_ONLY)
	} 
}

MENU   service_lv_1 {
	LABEL 	"|en|distance" ;
	ITEMS { 
		parMamFilterLength(READ_ONLY),
		parMamFilterBorder(READ_ONLY),
		parLowPassFilter(READ_ONLY),
		parHystereseWidth(READ_ONLY),
		parEchoLostRampUp(READ_ONLY),
		parEchoLostRampDown(READ_ONLY),
		parUnfilteredDistanceRaw(READ_ONLY)
	} 
}

MENU   service_lv_2 {
	LABEL 	"|en|envelope" ;
	ITEMS { 
		parHKStatistik(READ_ONLY),
		parFilterBreite(READ_ONLY),
		parEnvelopeEnergy(READ_ONLY)
	} 
}

MENU   service_lv_3 {
	LABEL 	"|en|mapping" ;
	ITEMS { 
		parMappingAdder(READ_ONLY),
		parMappingWidth(READ_ONLY),
		parMappingAverage(READ_ONLY),
		parFactoryTdt(READ_ONLY),
		parFacAdder(READ_ONLY),
		parFACWidth(READ_ONLY),
		parRangeFactoryMap(READ_ONLY),
		parFactoryMappingValid(READ_ONLY),
		parCustomerMappingValid(READ_ONLY)
	} 
}

MENU   service_lv_4 {
	LABEL 	"|en|edge" ;
	ITEMS { 
		parEdgeDetectionMode(READ_ONLY),
		parEdgeParameter(READ_ONLY),
		parCalcEdgeParameter(READ_ONLY)
	} 
}

MENU   service_lv_5 {
	LABEL 	"|en|first echo" ;
	ITEMS { 
		software_revision(HIDDEN),
		parFirstEchoFactor(READ_ONLY),
		parFefThreshold(READ_ONLY),
		parFefNearFactor(READ_ONLY),
		parFefDistanceNear(READ_ONLY),
		parFefDistanceFar(READ_ONLY),
		parFefEdge(READ_ONLY),
		parMaxEchoSN(READ_ONLY),
		parCurrentFirstEchoFactor(READ_ONLY)
	} 
}

MENU   service_lv_6 {
	LABEL 	"|en|tank bottom" ;
	ITEMS { 
		parDishBottomDetection(READ_ONLY),
		parAmplLevelBottomDetection(READ_ONLY),
		parMinLevelBottomDetection(READ_ONLY),
		parLevelSpeed(READ_ONLY)
	} 
}

MENU   service_lv_7 {
	LABEL 	"|en|module" ;
	ITEMS { 
		parHfModul(READ_ONLY),
		parZeroDistance(READ_ONLY)
	} 
}

MENU   service_lv_8 {
	LABEL 	"|en|antenna table" ;
	ITEMS { 
		software_revision(HIDDEN),
		parAntennaType(READ_ONLY),
		parInactiveLength(READ_ONLY),
		parTotalLength(READ_ONLY),
		parAbsMaxMessDistance(READ_ONLY),
		parPositionOfReferencePulse(READ_ONLY),
		parAmplOfReferencePulse(READ_ONLY),
		parEditRangeMaxSampleDist(READ_ONLY)
	} 
}

MENU   service_lv_12 {
	LABEL 	"|en|system 1" ;
	ITEMS { 
		software_revision(HIDDEN),
		parPlausibilityFunction(READ_ONLY),
		parMaxDeltaPlausibilityFunction(READ_ONLY),
		parUpperAreaPlausibilityFunction(READ_ONLY),
		parWireType(READ_ONLY),
		parDisplayVersion(READ_ONLY),
		parCalculatedCycleTime(READ_ONLY),
		parSwVersion(READ_ONLY),
		parParamVersion(READ_ONLY)
	} 
}

MENU   service_lv_13 {
	LABEL 	"|en|system 2" ;
	ITEMS { 
		software_revision(HIDDEN),
		parDAAdjust4mA(READ_ONLY),
		parDAAdjust20mA(READ_ONLY),
		parDeviceIDNumber(READ_ONLY)
	} 
}

MENU   service_lv_15 {
	LABEL 	"|en|debug" ;
	ITEMS { 
		parDaaIndex1(READ_ONLY),
		parDaaIndex2(READ_ONLY),
		parDaaRdWrValue(READ_ONLY),
		parErrorStateOfAlgorithm(READ_ONLY),
		parStateOfAlgorithm(READ_ONLY)
	} 
}

MENU   mnuEhIdent {
	LABEL 	[menu_table_sensor_level_radar] ;
	ITEMS { 
		mnuUnivIdent
	} 
}

MENU   mnuUnivIdent {
	LABEL 	[menu_ident] ;
	ITEMS { 
		Tab_ident_opunit_universal,
		Tab_ident_device
	} 
}

MENU   Tab_ident_opunit_universal {
	LABEL 	[menu_ident_opunit] ;
	ITEMS { 
		tag,
		descriptor,
		message
	} 
}

MENU   Tab_ident_device {
	LABEL 	[menu_ident_device] ;
	ITEMS { 
		manufacturer_id(READ_ONLY),
		device_type(READ_ONLY),
		device_id(READ_ONLY),
		universal_revision(READ_ONLY),
		transmitter_revision(READ_ONLY),
		software_revision(READ_ONLY),
		hardware_revision(READ_ONLY),
		device_flags(READ_ONLY),
		physical_signaling_code(READ_ONLY),
		final_assembly_number,
		write_protect(READ_ONLY),
		date
	} 
}


MENU   Menu_Main_Maintenance {
	LABEL 	[main_menu] ;
	ITEMS { 
		Menu_File,
		Menu_S_Device,
		Menu_S_View,
		Menu_S_Options,
		Menu_Help
	} 
}

MENU   Table_Main_Maintenance {
	LABEL 	DEFAULT_VALUE_phys_device_id ;
	ITEMS { 
		MenuMainMicroPilot_M,
		mnuEhIdent
	} 
}


/* !Collections! ***********************************************************************/


COLLECTION OF VARIABLE meas_val_unit
{
	LABEL		"MEAS_VAL_UNIT" ;
    MEMBERS
    {
        UNITS, parCustomerUnit,        [units] ;
        VALUE, parMeasuredValue,       [blank] ;
    }
}

COLLECTION OF VARIABLE collectionTag
{
        MEMBERS
        {
                parTagMPM, tag, "|en|tag no.|de|Messstelle|fr|N Repere|es|Messstelle|it|Tag n|nl|tag nummer";
        }
}

/************************************************************************************************************/
/*                         ARRAY`s									    */
/************************************************************************************************************/



ARRAY OF VARIABLE hart_var	/* use because of upper/lower case or E+H defines own text */
{
	ELEMENTS
    {
            /*   No.  Identifier    HART-text definition           */
            /* 0,  tag,                          [tag] ;*/
        1,  device_id,        [device_identification] ;
    }
}


VARIABLE variable_array_index
{
    LABEL   "VARIABLE_ARRAY_INDEX" ;
    TYPE    INDEX (2) variable_array_val;
    CLASS   USER_INTERFACE ;
    HANDLING READ & WRITE ;
}

/************************************************************************************************************/
/*                         ARRAY OF VARIABLE / MATRIX DEFINITION								        */
/************************************************************************************************************/

ARRAY OF VARIABLE variable_array_val
{
	LABEL		"VARIABLE_ARRAY_VALUE" ;
    ELEMENTS
    {
         59, parMeasuredValue ;           /*VH00*/
         36, parTankShape;                    /*VH02*/
        202, parDielectricConstant;               /*VH03*/
        203, parProcessCondition;             /*VH04*/
         45, parEmptyCalibration ;            /*VH05*/
         62, parFullCalibration ;         /*VH06*/
         83, parTubeDiameter ;            /*VH07*/
         34, parOutputOnAlarm;            /*VH10*/
         52, parOutputAtError;            /*VH11*/
        107, parReactionLostEcho;         /*VH12*/
        122, parRampInPercentPerMin;          /*VH13*/
         53, parDelayTimeOnLostEcho;          /*VH14*/
        109, parLevelWithinSafetyDistance;        /*VH15*/
         88, parInSafetyDistance;         /*VH16*/
         68, parAcknowledgeAlarm;         /*VH17*/
         46, parOperationMode;            /*VH18*/


        126, parLevelUllageMode;          /*VH30*/
         55, parLinearisation;            /*VH31*/
         26, parCustomerUnit;             /*VH32*/
         61, parTableNumber;              /*VH33*/
         58, parInputLevelManual;         /*VH34*/
         57, parInputLevelHalfAutomatic;       /*VH34*/
         60, parInputVolume;              /*VH35*/
         56, parMaxVolume;                /*VH36*/
        121, parCylinderVessel;           /*VH37*/

         /*39, parSelection;                  VH40*/
        244, parCheckDistance;            /*VH41*/
         82, parSuppressionDistance;          /*VH42*/
        110, parStartMappingRecord;           /*VH43*/
        245, parPresMapRange;                 /*VH44*/
        246, parCustTankMap;              /*VH45*/
         38, parEchoQuality;              /*VH46*/
         98, parOffsetOfMeasuredDistance;             /*VH47*/
         42, parOutputDamping;            /*VH48*/
         47, parBlockingDistance;         /*VH49*/

         13, parCommAddress;              /*VH50*/
         14, parPreambles;                /*VH51*/
        123, parThresholdMainValue;           /*VH52*/
        206, parFixedCurrentOutput;           /*VH53*/
        207, parFixedCurrentValue;            /*VH54*/
         33, parSimulation;               /*VH55*/
         66, parSimulationValueLevel;         /*VH56*/
         67, parSimulationValueVolume;        /*VH56*/
         37, parSimulationValueCurrent;       /*VH56*/
         54, parCurrentOutput;            /*VH57*/

        118, parLanguage;             /*VH62*/
        117, parBackToHome;               /*VH63*/
        108, parFormatDisplay_ft;         /*VH64*/
        114, parNoOfDecimals;             /*VH65*/
         43, parSeparationCharacter;          /*VH66*/
        /*105, parDisplayTest;                VH??*/

	     /*--ServiceLevel 0--*/
        127, parCurrentAmplitude;         /*VH70_1*/
        211, parAmplitudeOverSuppression;                /*VH70_2*/
        205, parAmplitudeOverFAC;         /*VH70_3*/
        134, parUnfilteredDistance;           /*VH70_4*/
         90, parCalcEdgeParameter;            /*VH70_5*/
        135, parCurrentFirstEchoFactor;       /*VH70_6*/
        247, parDeviceName;               /*VH70_8*/
        193, parOrderCode;                /*VH70_9*/

	     /*--ServiceLevel 1--*/
         91, parMamFilterLength;          /*VH71_1*/
         92, parMamFilterBorder;          /*VH71_2*/
        146, parLowPassFilter;            /*VH71_3*/
         95, parHystereseWidth;           /*VH71_4*/
        153, parEchoLostRampUp;           /*VH71_5*/
        154, parEchoLostRampDown;         /*VH71_6*/
        155, parUnfilteredDistanceRaw;        /*VH71_7*/

	     /*--ServiceLevel 2--*/
         94, parHKStatistik;              /*VH72_1*/
         96, parFilterBreite;             /*VH72_2*/
         87, parEnvelopeEnergy;           /*VH72_9*/

	     /*--ServiceLevel 3--*/
         136, parMappingAdder;             /*VH73_1*/
         138, parMappingWidth;             /*VH73_2*/
         162, parMappingAverage;           /*VH73_3*/
         102, parFactoryTdt;               /*VH73_4*/
         140, parFacAdder;             /*VH73_5*/
         139, parFACWidth;             /*VH73_6*/
         366, parRangeFactoryMap;          /*VH73_7*/
         142, parFactoryMappingValid;          /*VH73_8*/
         143, parCustomerMappingValid;         /*VH73_9*/

         /*--ServiceLevel 4--*/
         187, parEdgeDetectionMode;            /*VH74_1*/
         145, parEdgeParameter;            /*VH74_2*/
         /*90, parCalcEdgeParameter;*/             /*VH74_9 already defined in VH70_5*/

	     /*--ServiceLevel 5--*/
         148, parFirstEchoFactor;          /*VH75_1*/
         171, parFefThreshold;             /*VH75_2*/
         172, parFefNearFactor;            /*VH75_3*/
         173, parFefDistanceNear;          /*VH75_4*/
         174, parFefDistanceFar;           /*VH75_5*/
         175, parFefEdge;              /*VH75_6*/
         /*135, parCurrentFirstEchoFactor;*/       /*VH75_9 already defined in VH70_6*/

	     /*--ServiceLevel 6--*/
         150, parDishBottomDetection;            /*VH76_1*/
         152, parAmplLevelBottomDetection;               /*VH76_2*/
         176, parMinLevelBottomDetection;                /*VH76_3*/
         177, parLevelSpeed;               /*VH76_9*/

	     /*--ServiceLevel 7--*/
          40, parHfModul;              /*VH77_1*/
         160, parZeroDistance;             /*VH77_2*/

	     /*--ServiceLevel 8--*/
         165, parAntennaType;              /*VH78_1*/
         166, parInactiveLength;           /*VH78_2*/
         167, parTotalLength;                  /*VH78_3*/
          85, parAbsMaxMessDistance;           /*VH78_4*/
         158, parPositionOfReferencePulse;     /*VH78_8*/
         159, parAmplOfReferencePulse;         /*VH78_9*/


	     /*--ServiceLevel 12--*/

         197, parPlausibilityFunction;             /*VH7C_1*/
         199, parMaxDeltaPlausibilityFunction;           /*VH7C_2*/
         200, parUpperAreaPlausibilityFunction;          /*VH7C_3*/
          78, parDisplayVersion;               /*VH7C_5*/
         186, parCalculatedCycleTime;              /*VH7C_6*/
         181, parSwVersion;                    /*VH7C_8*/
         204, parParamVersion;                 /*VH7C_9*/

	     /*--ServiceLevel 13--*/

          63, parDAAdjust4mA;                      /*VH7D_1*/
         120, parDAAdjust20mA;                     /*VH7D_2*/

	     /*--ServiceLevel 15--*/

          64, parDaaIndex1;                        /*VH7F_1*/
          65, parDaaIndex2;                        /*VH7F_2*/
          80, parDaaRdWrValue;                     /*VH7F_3*/
         132, parErrorStateOfAlgorithm;                /*VH7F_7*/
         133, parStateOfAlgorithm;                 /*VH7F_8*/

           2, parActualError;              /*VH90*/
           3, parLastError;                /*VH91*/
         111, parClearLastError;           /*VH91*/
          35, parReset;                /*VH93*/
           0, parOperationCode;            /*VH94*/
          44, parMeasuredDistance;         /*VH95*/
          48, parMeasuredLevel;            /*VH96*/
         250, parApplicationParameter;         /*VH91*/

         371, parTagNumber;                /*VHA0*/
          81, parProtSoftVersionString;        /*VHA2*/
         192, parSerialNumber;             /*VHA4*/
          25, parDistanceUnit;                 /*VHA5*/
         130, parDownloadMode;                 /*VHA8*/

         // (JM) : new variables for Software Revision 2

         460, parAntennaExtensionLength;
         448, parMaxEchoSN;
         459, parEditRangeMaxSampleDist;
         449, parWireType;
         461, parDeviceIDNumber;
         450, parCurrentOutputMode;
    }
}

/************************************************************************************************************/
/*                         ARRAY OF VARIABLE / UNIT DEFINITION												*/
/************************************************************************************************************/

ARRAY OF VARIABLE variable_array_unit
{
	LABEL		"VARIABLE_ARRAY_UNIT" ;
	ELEMENTS
    {
         59, parCustomerUnit;             /*VH00*/
         36, nul_unit;                    /*VH02*/
        202, nul_unit;                    /*VH03*/
        203, nul_unit;                    /*VH04*/
         45, parDistanceUnit ;            /*VH05*/
         62, parDistanceUnit ;            /*VH06*/
         83, parDistanceUnit ;            /*VH07*/

         34, nul_unit;                /*VH10*/
         52, nul_unit;                /*VH11*/
        107, nul_unit;                /*VH12*/
        122, nul_unit;                /*VH13*/
         53, nul_unit;                /*VH14*/
        109, parDistanceUnit;             /*VH15*/
         88, nul_unit;                /*VH16*/
         68, nul_unit;                /*VH17*/
         46, nul_unit;                /*VH18*/


        126, nul_unit;                /*VH30*/
         55, nul_unit;                /*VH31*/
         26, nul_unit;                /*VH32*/
         61, nul_unit;                /*VH33*/
         58, parDistanceUnit;             /*VH34*/
         57, parDistanceUnit;             /*VH34*/
         60, parDistanceUnit;             /*VH35*/
         56, parCustomerUnit;             /*VH36*/
        121, parDistanceUnit;             /*VH37*/

         39, nul_unit;                                /*VH40*/
        244, nul_unit;                /*VH41*/
         82, parDistanceUnit;             /*VH42*/
        110, nul_unit;                /*VH43*/
        245, parDistanceUnit;                 /*VH44*/
        246, nul_unit;                /*VH45*/
         38, nul_unit;                /*VH46*/
         98, parDistanceUnit;             /*VH47*/
         42, nul_unit;                /*VH48*/
         47, parDistanceUnit;             /*VH49*/

         13, nul_unit;                /*VH50*/
         14, nul_unit;                /*VH51*/
        123, nul_unit;                /*VH52*/
        206, nul_unit;                /*VH53*/
        207, nul_unit;                /*VH54*/
         33, nul_unit;                /*VH55*/
         66, parDistanceUnit;             /*VH56*/
         67, parCustomerUnit;             /*VH56*/
         37, nul_unit;                /*VH56*/
         54, parCurrentOutput;            /*VH57*/

        118, nul_unit;                /*VH62*/
        117, nul_unit;                /*VH63*/
        108, nul_unit;                /*VH64*/
        114, nul_unit;                /*VH65*/
         43, nul_unit;                /*VH66*/
        105, nul_unit;                /*VH??*/

	     /*--ServiceLevel 0--*/
        127, nul_unit;                /*VH70_1*/
        211, nul_unit;                /*VH70_2*/
        205, nul_unit;                /*VH70_3*/
        134, nul_unit;                /*VH70_4*/
         90, nul_unit;                /*VH70_5*/
        135, nul_unit;                /*VH70_6*/
        247, nul_unit;                /*VH70_8*/
        193, nul_unit;                /*VH70_9*/

	     /*--ServiceLevel 1--*/
         91, nul_unit;                /*VH71_1*/
         92, nul_unit;                /*VH71_2*/
        146, nul_unit;                /*VH71_3*/
         95, parDistanceUnit;             /*VH71_4*/
        153, parDistanceUnit;             /*VH71_5*/
        154, parDistanceUnit;             /*VH71_6*/
        155, parDistanceUnit;             /*VH71_7*/
        156, parDistanceUnit;             /*VH71_8*/
        157, parDistanceUnit;             /*VH71_9*/

	     /*--ServiceLevel 2--*/
         94, nul_unit;                /*VH72_1*/
         96, nul_unit;                /*VH72_2*/
         87, nul_unit;                /*VH72_9*/

	     /*--ServiceLevel 3--*/
        136, nul_unit;                /*VH73_1*/
        138, nul_unit;                /*VH73_2*/
        162, nul_unit;                /*VH73_3*/
        102, nul_unit;                /*VH73_4*/
        140, nul_unit;                /*VH73_5*/
        139, nul_unit;                /*VH73_6*/
        366, parDistanceUnit;             /*VH73_7*/
        142, nul_unit;                /*VH73_8*/
        143, nul_unit;                /*VH73_9*/

	     /*--ServiceLevel 4--*/
        187, nul_unit;                /*VH74_1*/
        145, nul_unit;                /*VH74_2*/
         /*90, parCalcEdgeParameter;              VH74_9 already defined in VH70_5*/

	     /*--ServiceLevel 5--*/
        148, nul_unit;                /*VH75_1*/
        171, nul_unit;                /*VH75_2*/
        172, nul_unit;                /*VH75_3*/
        173, parDistanceUnit;             /*VH75_4*/
        174, parDistanceUnit;             /*VH75_5*/
        175, nul_unit;                /*VH75_6*/
        /*135, parCurrentFirstEchoFactor;     VH75_9 already defined in VH70_6*/

	     /*--ServiceLevel 6--*/
        150, nul_unit;                /*VH76_1*/
        152, nul_unit;                /*VH76_2*/
        176, parDistanceUnit;             /*VH76_3*/
        177, parDistanceUnit;             /*VH76_9*/

	     /*--ServiceLevel 7--*/
         40, nul_unit;                /*VH77_1*/
        160, parDistanceUnit;             /*VH77_2*/

	     /*--ServiceLevel 8--*/
        165, nul_unit;                /*VH78_1*/
        166, parDistanceUnit;             /*VH78_2*/
        167, parDistanceUnit;             /*VH78_3*/
         85, parDistanceUnit;             /*VH78_4*/
        158, parDistanceUnit;             /*VH78_8*/
        159, nul_unit;                /*VH78_8*/


	     /*--ServiceLevel 12--*/

        197, nul_unit;                    /*VH7C_1*/
        199, parDistanceUnit;                 /*VH7C_2*/
        200, parDistanceUnit;                 /*VH7C_3*/
         78, nul_unit;                    /*VH7C_5*/
        186, nul_unit;                    /*VH7C_6*/
        181, nul_unit;                    /*VH7C_8*/
        204, nul_unit;                    /*VH7C_9*/

	     /*--ServiceLevel 13--*/

         63, nul_unit;                        /*VH7D_1*/
        120, nul_unit;                        /*VH7D_2*/

	     /*--ServiceLevel 15--*/

         64, nul_unit;                        /*VH7F_1*/
         65, nul_unit;                        /*VH7F_2*/
         80, nul_unit;                        /*VH7F_3*/
        132, nul_unit;                        /*VH7F_7*/
        133, nul_unit;                        /*VH7F_8*/

          2, nul_unit;                /*VH90*/
          3, nul_unit;                /*VH91*/
        111, nul_unit;                /*VH91*/
         35, nul_unit;                /*VH93*/
          0, nul_unit;                /*VH94*/
         44, parDistanceUnit;             /*VH95*/
         48, parDistanceUnit;             /*VH96*/
        250, nul_unit;                /*VH91*/

        371, nul_unit;                /*VHA0*/
         81, nul_unit;                /*VHA2*/
        192, nul_unit;                /*VHA4*/
         25, nul_unit;                    /*VHA5*/
        130, nul_unit;                    /*VHA8*/

         // (JM) : new variables-units for Software Revision 2

         460, parDistanceUnit;
         448, nul_unit;
         459, parDistanceUnit;
         449, nul_unit;
         461, nul_unit;
         450, nul_unit;
    }
}


VARIABLE format_table_index
{
	LABEL		"FORMAT_TABLE_INDEX" ;
	TYPE		INDEX format_table ;
	CLASS		USER_INTERFACE ;
}

ARRAY OF VARIABLE format_table
{
	LABEL		"FORMAT_TABLE" ;
	ELEMENTS	{
					0x00, format ;
					0x58, format ;
					0x59, format ;
				}
}


/************************************************************************************************************/
/*                         DYNAMIC VARIABLES																*/
/************************************************************************************************************/

COLLECTION OF VARIABLE var_collection_level_1
{
	LABEL		[level] ;
    MEMBERS
    {
            /* Measurement */

        DIGITAL_UNITS,     parCustomerUnit,              [units] ;
        DIGITAL_VALUE,     parMeasuredValue,             [blank] ;
        ANALOG_VALUE,      parCurrentOutput,             [analog_output_value] ;
        PERCENT_RANGE,     percent_range,               "%-RANGE" ;

        /* Sensor Information */

        SENSOR_UNITS,          parDistanceUnit,          [sensor_unit] ;
        UPPER_SENSOR_LIMIT,    sensor_upper_limit,       [usl] ;
        LOWER_SENSOR_LIMIT,    sensor_lower_limit,       [lsl] ;
        MINIMUM_SPAN,          sensor_minimum_span,      [minimum_span] ;
        SENSOR_SERIAL_NUMBER,  sensor_serial_number,     [sensor_serial_number] ;

        /* Output Information */

        ALARM_CODE,           parOutputOnAlarm,          [alarm_select] ;
        TRANSFER_FUNCTION,    output_transfer_function,  [transfer_function] ;
        RANGE_UNITS,          parCustomerUnit,           [range_units] ;
        UPPER_RANGE_VALUE,    dummy_float,               [urv] ;
        LOWER_RANGE_VALUE,    dummy_float,               [lrv] ;
        DAMPING_VALUE,        parOutputDamping,          [damping_value] ;
    }
}

ARRAY OF COLLECTION dynamic_variables
{
    ELEMENTS
    {
        PRIMARY, var_collection_level_1, [pv], [primary_variable_help] ;
    }
}



/* !Commands! **************************************************************************/
COMMAND read_display_format
{
	NUMBER		142 ;
	OPERATION	READ ;
	TRANSACTION	{
			 REQUEST
			 {
			  format_table_index(INFO,INDEX)
			 }
			 REPLY
			 {
			  response_code ,
			  device_status ,
			  format_table_index ,
			  format_table[format_table_index]
			 }
			}

	RESPONSE_CODES	{
			 0, SUCCESS,          [no_command_specific_errors] ;
			 2, DATA_ENTRY_ERROR, [invalid_selection] ;
			 5, MISC_ERROR,       [too_few_data_bytes_recieved] ;
			}
}



COMMAND read_index_parameter
{
	NUMBER		174 ;
	OPERATION	READ ;
	TRANSACTION	{
				 	REQUEST
				 	{

				  		variable_array_index(INFO,INDEX )

				 	}
			          REPLY
				 	{
					  response_code ,
					  device_status ,
					  variable_array_val[variable_array_index]
				 	}
				}

				RESPONSE_CODES
				{
					 0, SUCCESS,          [no_command_specific_errors] ;
					 2, DATA_ENTRY_ERROR, [invalid_selection] ;
					 5, MISC_ERROR,       [too_few_data_bytes_recieved] ;

				}
}

COMMAND write_index_parameter
{
	NUMBER		175 ;
	OPERATION	WRITE ;
	TRANSACTION	{
			 		REQUEST
			 		{
						  variable_array_index( INFO,INDEX ) ,
						  variable_array_val[variable_array_index]
			 		}
			 		REPLY
			 		{
						  response_code ,
						  device_status ,
						  variable_array_val[variable_array_index]
			 		}
				}

				RESPONSE_CODES
				{
					 0, SUCCESS,          [no_command_specific_errors] ;
					 2, DATA_ENTRY_ERROR, [invalid_selection] ;
					 3, DATA_ENTRY_ERROR, [passed_parameter_too_large] ;
					 4, DATA_ENTRY_ERROR, [passed_parameter_too_small] ;
					 5, MISC_ERROR,       [too_few_data_bytes_recieved] ;
					 7, MODE_ERROR,       [in_write_protect_mode] ;
					 8, MISC_WARNING,     "Set to nearest possible value" ;
					 9, DATA_ENTRY_ERROR, "Invalid value selection" ;
					16, MODE_ERROR,       [access_restricted] ;
				}
}


/* !Methods! ***************************************************************************/
METHOD assignDistanceUnit
{
        preDistanceUnit = parDistanceUnit;
}


METHOD nonMetricToMetricUnits
{
        DEFINITION
        {
                fassign(parTubeDiameter, fvar_value(parTubeDiameter)*25.4);
                fassign(parUnfilteredDistance, fvar_value(parUnfilteredDistance)*25.4);
                fassign(parHystereseWidth, fvar_value(parHystereseWidth)*25.4);
                fassign(parEchoLostRampUp, fvar_value(parEchoLostRampUp)*25.4);
                fassign(parEchoLostRampDown, fvar_value(parEchoLostRampDown)*25.4);
                fassign(parUnfilteredDistanceRaw, fvar_value(parUnfilteredDistanceRaw)*25.4);
                fassign(parFefDistanceNear, fvar_value(parFefDistanceNear)*25.4);
                fassign(parFefDistanceFar, fvar_value(parFefDistanceFar)*25.4);
                fassign(parMinLevelBottomDetection, fvar_value(parMinLevelBottomDetection)*25.4);
                fassign(parLevelSpeed, fvar_value(parLevelSpeed)*25.4);
                fassign(parZeroDistance, fvar_value(parZeroDistance)*25.4);
                fassign(parInactiveLength, fvar_value(parInactiveLength)*25.4);
                fassign(parTotalLength, fvar_value(parTotalLength)*25.4);
                fassign(parPositionOfReferencePulse, fvar_value(parPositionOfReferencePulse)*25.4);
                fassign(parMaxDeltaPlausibilityFunction, fvar_value(parMaxDeltaPlausibilityFunction)*25.4);
                fassign(parUpperAreaPlausibilityFunction, fvar_value(parUpperAreaPlausibilityFunction)*25.4);
        }
}

METHOD metricToNonMetricUnits
{
        DEFINITION
        {
                fassign(parTubeDiameter, fvar_value(parTubeDiameter)/25.4);
                fassign(parUnfilteredDistance, fvar_value(parUnfilteredDistance)/25.4);
                fassign(parHystereseWidth, fvar_value(parHystereseWidth)/25.4);
                fassign(parEchoLostRampUp, fvar_value(parEchoLostRampUp)/25.4);
                fassign(parEchoLostRampDown, fvar_value(parEchoLostRampDown)/25.4);
                fassign(parUnfilteredDistanceRaw, fvar_value(parUnfilteredDistanceRaw)/25.4);
                fassign(parFefDistanceNear, fvar_value(parFefDistanceNear)/25.4);
                fassign(parFefDistanceFar, fvar_value(parFefDistanceFar)/25.4);
                fassign(parMinLevelBottomDetection, fvar_value(parMinLevelBottomDetection)/25.4);
                fassign(parLevelSpeed, fvar_value(parLevelSpeed)/25.4);
                fassign(parZeroDistance, fvar_value(parZeroDistance)/25.4);
                fassign(parInactiveLength, fvar_value(parInactiveLength)/25.4);
                fassign(parTotalLength, fvar_value(parTotalLength)/25.4);
                fassign(parPositionOfReferencePulse, fvar_value(parPositionOfReferencePulse)/25.4);
                fassign(parMaxDeltaPlausibilityFunction, fvar_value(parMaxDeltaPlausibilityFunction)/25.4);
                fassign(parUpperAreaPlausibilityFunction, fvar_value(parUpperAreaPlausibilityFunction)/25.4);
        }
}


METHOD Reset_Config_Changed_Flag
{
        LABEL           [Online_Reset_Config_Changed_Flag];
	DEFINITION	{
                        ACKNOWLEDGE ("|en|Do you want to reset the 'Config Changed Flag' ?|de|Wollen Sie den Konfigurationsmerker zurcksetzen ?");
                        send_command(38);
			}
}

METHOD Restart_Device
{
        LABEL           "Restart Device" ;
	DEFINITION	{
                        ACKNOWLEDGE ("Attention !   Device will be restarted !\n \n \n Loss of information possible !");
                        send_command(42);
			}
}

METHOD method_set
{
	LABEL       [transfer];
	HELP        [transfer_help];
	DEFINITION
	{
	}
}

METHOD uploadVariablesPreWriteActions
{
        LABEL "Download of the Operation Code before uploading the variables" ;
        DEFINITION
        {
                // (JM) Command "id_read_unique_identifier"
                // Read in particular the variable "software_revision"
                send_command(0);

                variable_array_index = 0;       // Ref to var: par_operation_code
                send_command(174);              // Read "par_operation_code"
                if (parShowAccess == 0)
                {
                        ACKNOWLEDGE("|en|No variables have been uploaded|de|Es wurde keine Variablen geschrieben");
                }
        }
}

METHOD downloadVariablesPreReadActions
{
        LABEL "downloadVariablesPreReadActions" ;
        DEFINITION
        {
                // (JM) Command "id_read_unique_identifier"
                // Read in particular the variable "software_revision"
                send_command(0);

                // Note (JM): This variable should be in "download menu", but in this case
                // the unit relations are (because of PDM) not run.
                variable_array_index = 25;
                send_command(174);
        }
}

/*
METHOD synchronizeTagToParTagNumber
{
        LABEL "Synchonize tag to parTagNumber" ;
        DEFINITION
        {
                parTagNumber = tag;
        }
}

METHOD synchronizeParTagNumberToTag
{
        LABEL "Synchonize parTagNumber to tag" ;
        DEFINITION
        {
                char myString[16];
                myString = parTagNumber;
                myString[8] = 0;                // Cut end of string
                tag = myString;
                parTagNumber = myString;
        }
}
*/

METHOD preOperationCodeModified
{
        LABEL "Copy value of preOperationCode onto parOperationCode" ;
        DEFINITION
        {
                parOperationCode  = preOperationCode;   // Not used anymore (perOperationMode is Hidden)
        }
}

METHOD updateParShowAccess
{
        LABEL "Update parShowAccess after preOperationCode gets modified" ;
        DEFINITION
        {
                switch (parOperationCode)
                {
                        default:
                        {
                                parShowAccess = 0;
                        }; break;
                        case 100:
                        {
                                parShowAccess = 1;
                        }; break;
                        case 200:
                        {
                                parShowAccess = 2;
                        }; break;
                        case 300:
                        {
                                parShowAccess = 3;
                        }; break;
                        case 33998:
                        {
                                parShowAccess = 5;
                        }; break;
                        case 123:
                        {
                                parShowAccess = 6;
                        }; break;
                        case 33999:
                        {
                                parShowAccess = 7;
                        }; break;

                }
        }
}

METHOD setAbsMax
{
        DEFINITION
        {
                iassign(parMaxAbs, 1);
        }
}

METHOD synchronizeParProcessConditionToParOutputDamping
{
        DEFINITION
        {
                switch (parProcessCondition)
                {
                        default:
                        case 0: // standard
                        case 2: // unruhig
                        case 3: // Rhrwerk
                        {
                                parOutputDamping = 3;
                        }; break;
                        case 1: // ruhig
                        {
                                parOutputDamping = 10;
                        }; break;
                        case 4: // schnell
                        case 5: // test (no filter)
                        {
                                parOutputDamping = 0;
                        }; break;
                }

        }
}

METHOD std_CommWay
{
     LABEL          [menu_Device_CommWay];
     DEFINITION
     {
     }
}

METHOD PostReadId_device_revision
{
        LABEL "PostReadId_device_revision" ;  // See also PostReadSoftware_revision
        DEFINITION
        {
                if (ivar_value(id_device_revision) == CHECK_SOFTWARE_VERSION)
                {
                        parSoftwareVersion = 1;
                }
                else if (ivar_value(id_device_revision) == CHECK_SOFTWARE_VERSION02)
                {
                        parSoftwareVersion = 2;
                }
                else
                {
                        parSoftwareVersion = 0;
                }
        }
}

METHOD PostReadSoftware_revision
{
        LABEL "PostReadSoftware_revision" ;  // See also PostReadId_device_revision
        DEFINITION
        {
                if (ivar_value(software_revision) == CHECK_SOFTWARE_VERSION)
                {
                        parSoftwareVersion = 1;
                }
                else if (ivar_value(software_revision) == CHECK_SOFTWARE_VERSION02)
                {
                        parSoftwareVersion = 2;
                }
                else
                {
                        parSoftwareVersion = 0;
                }
        }
}



/* !Relations! *************************************************************************/
UNIT parCustomerUnit_relation
{
	parCustomerUnit:
					parMeasuredValue,
					parInputVolume,
					parMaxVolume,
					parSimulationValueVolume
}

UNIT parDistanceUnit_relation
{
        parDistanceUnit:
                                        parAbsMaxMessDistance,
                                        parEmptyCalibration,
                                        parFullCalibration ,
                                        parCylinderVessel,
                                        parPresMapRange,
                                        parRangeFactoryMap,
                                        parLevelWithinSafetyDistance,
                                        parMeasuredDistance,
                                        parInputLevelManual,
                                        parInputLevelHalfAutomatic,
                                        parSuppressionDistance,
                                        parOffsetOfMeasuredDistance,
                                        parBlockingDistance,
                                        parSimulationValueLevel,
                                        parMeasuredLevel,
                                        parAntennaExtensionLength
}

MENU upload_variables { 
	LABEL "|en|Upload variables";
	PRE_WRITE_ACTIONS {
		uploadVariablesPreWriteActions
	}
	ITEMS { 
		IF ( Hnd000 ) { 
			parDistanceUnit
		}
		IF ( Hnd000 ) { 
			parTankShape,
			parDielectricConstant,
			parProcessCondition,
			parEmptyCalibration,
			parFullCalibration,
			parFormatDisplay_ft,
			parOutputOnAlarm,
			parReactionLostEcho,
			parDelayTimeOnLostEcho,
			parLevelWithinSafetyDistance,
			parInSafetyDistance,
			parOffsetOfMeasuredDistance,
			parOutputDamping,
			parBlockingDistance,
			parBackToHome,
			parClearLastError,
			parTagNumber
		}

		IF ( Hnd009 ) { 
			parLanguage
		}

		IF ( Hnd011 ) { 
			tag,
			message,
			descriptor,
			date,
			final_assembly_number
		}

		IF ( Hnd013 ) { 
			parAcknowledgeAlarm
		}

		IF ( Hnd000 ) {

			IF ( Val005 ) { 
				parNoOfDecimals,
				parSeparationCharacter
			}

			IF ( Val007 ) { 
				parTubeDiameter
			}

			IF ( Val009 ) { 
				parRampInPercentPerMin
			}

			IF ( Val016 ) { 
				parAntennaExtensionLength,
				parCurrentOutputMode
			}
		}

		IF ( Hnd003 ) {

			IF ( Val016 ) { 
				parDeviceIDNumber
			}
		}
	}
}
MENU download_variables { 
	LABEL [menu_load_to_PC] ; 
	PRE_READ_ACTIONS {
		downloadVariablesPreReadActions
	}
	ITEMS {
		parOperationCode,
		parCustomerUnit,
		parAbsMaxMessDistance,
		parMeasuredValue,
		parNoOfDecimals,
		parTankShape,
		parDielectricConstant,
		parProcessCondition,
		parEmptyCalibration,
		parFullCalibration,
		parOperationMode,
		parFormatDisplay_ft,
		parOutputOnAlarm,
		parReactionLostEcho,
		parTubeDiameter,
		parOutputAtError,
		parRampInPercentPerMin,
		parLevelUllageMode,
		parLinearisation,
		parDelayTimeOnLostEcho,
		parLevelWithinSafetyDistance,
		parInSafetyDistance,
		parAcknowledgeAlarm,
		parTableNumber,
		parInputLevelHalfAutomatic,
		parInputLevelManual,
		parInputVolume,
		parMaxVolume,
		parCylinderVessel,
		parCheckDistance,
		parSuppressionDistance,
		parStartMappingRecord,
		parPresMapRange,
		parCustTankMap,
		parEchoQuality,
		parOffsetOfMeasuredDistance,
		parOutputDamping,
		parBlockingDistance,
		parCommAddress,
		parPreambles,
		parThresholdMainValue,
		parFixedCurrentValue,
		parSimulation,
		parSimulationValueLevel,
		parSimulationValueCurrent,
		parCurrentOutput,
		parLanguage,
		parBackToHome,
		parSeparationCharacter,
		parCurrentAmplitude,
		parAmplitudeOverSuppression,
		parAmplitudeOverFAC,
		parUnfilteredDistance,
		parCalcEdgeParameter,
		parCurrentFirstEchoFactor,
		parDeviceName,
		parOrderCode,
		parMamFilterLength,
		parMamFilterBorder,
		parLowPassFilter,
		parHystereseWidth,
		parEchoLostRampUp,
		parEchoLostRampDown,
		parUnfilteredDistanceRaw,
		parHKStatistik,
		parFilterBreite,
		parEnvelopeEnergy,
		parMappingAdder,
		parMappingWidth,
		parMappingAverage,
		parFactoryTdt,
		parFacAdder,
		parFACWidth,
		parRangeFactoryMap,
		parFactoryMappingValid,
		parCustomerMappingValid,
		parEdgeDetectionMode,
		parEdgeParameter,
		parFirstEchoFactor,
		parFefThreshold,
		parFefNearFactor,
		parFefDistanceNear,
		parFefDistanceFar,
		parFefEdge,
		parDishBottomDetection,
		parAmplLevelBottomDetection,
		parMinLevelBottomDetection,
		parLevelSpeed,
		parHfModul,
		parZeroDistance,
		parAntennaType,
		parInactiveLength,
		parTotalLength,
		parPositionOfReferencePulse,
		parAmplOfReferencePulse,
		parPlausibilityFunction,
		parMaxDeltaPlausibilityFunction,
		parUpperAreaPlausibilityFunction,
		parDisplayVersion,
		parCalculatedCycleTime,
		parSwVersion,
		parParamVersion,
		parDAAdjust4mA,
		parDAAdjust20mA,
		parDaaIndex1,
		parDaaIndex2,
		parDaaRdWrValue,
		parErrorStateOfAlgorithm,
		parStateOfAlgorithm,
		parActualError,
		parLastError,
		parClearLastError,
		parMeasuredDistance,
		parMeasuredLevel,
		parApplicationParameter,
		parTagNumber,
		parProtSoftVersionString,
		parSerialNumber,
		parDownloadMode,
		parSimulationValueVolume,
		manufacturer_id,
		device_type,
		universal_revision,
		transmitter_revision,
		software_revision,
		hardware_revision,
		physical_signaling_code,
		device_flags,
		device_id,
		tag,
		message,
		descriptor,
		date,
		write_protect,
		final_assembly_number,
		device_status
		IF ( Dlc000 ) { 
			parAntennaExtensionLength,
			parMaxEchoSN,
			parEditRangeMaxSampleDist,
			parWireType,
			parDeviceIDNumber,
			parCurrentOutputMode
		}
		IF ( Dlc002 ) { 
			parFixedCurrentOutput
		}

	} 
} 

